///////////////////////////////////////////////////////
//
// Main.cpp
//
// This file is analagous to the "application" code
// described in the article (vs. the library code).
// Notice that we create and use the TIFF converter 
// without ever including TiffCvt.h.
//
///////////////////////////////////////////////////////

#include <stdio.h>
#include <vector>
#include "Store.h"
#include "Cvt.h"
#include "CvtProxy.h"

//
// Declare a single instance of the converter store.
//
FileConverterStore gConverterStore;

//
// Walk the list of classes that the Specialty
// Store knows about and print the extension
// for all compressed formats.
//
void PrintCompressedFormats()
{
    int size = gConverterStore.GetCount();
    const FileConverterProxyBase* pCvt;

    printf("Compressed formats:\n" );
    for (int i=0; i<size; i++)
    {
        pCvt = gConverterStore.GetAt(i);
        if ( pCvt->IsCompressed() )
            printf("%s\n", pCvt->GetExtension() );
    }
} 

int main(void)
{
    PrintCompressedFormats();

    printf("Creating and calling converter for .TIF:\n" );
    FileConverter* pConverter;
    pConverter = gConverterStore.CreateByExtension(".TIF");
    if (pConverter)
        pConverter->Import("Image.TIF");
    delete pConverter;

	// Make the application wait before closing the window.
    getchar();
    return 0;
}
