///////////////////////////////////////////////////////
//
// Store.cpp
//
// Copyright 1997, James E. Beveridge.
// Permission is granted to freely reuse this code
// as long as credit is given in your source code.
//
// Implementation of a specialty store for
// file converters.
//
///////////////////////////////////////////////////////

#include <stdio.h>
#include <vector>
#include "Store.h"
#include "Cvt.h"
#include "CvtProxy.h"


//////////////////////////////////////////////
// FileConverterStore member functions

FileConverterStore::FileConverterStore()
{
    // Must be empty - Register() will
    // probably already have been called
    // by the time the compiler gets around
    // to calling this function.
}

FileConverterStore::~FileConverterStore()
{
    // Individual entries in this vector
    // do not need to be deleted.
    delete m_pConverters;
}

void FileConverterStore::Init()
{
    if (!m_pConverters)
        m_pConverters = new ConverterVector;
}

void FileConverterStore::Register(const FileConverterProxyBase* proxy)
{
    Init();
    m_pConverters->push_back(proxy);
}

int FileConverterStore::GetCount()
{
    if (!m_pConverters)
        return 0;
    return m_pConverters->size();
}

const FileConverterProxyBase* FileConverterStore::GetAt(int i)
{
    return m_pConverters->at(i);
}

FileConverter* FileConverterStore::CreateByExtension(const char* ext)
{
    for (unsigned i=0; i< m_pConverters->size(); i++)
        if ( stricmp( m_pConverters->at(i)->GetExtension(), ext ) == 0 )
            return m_pConverters->at(i)->CreateObject();
    return NULL;
}
