///////////////////////////////////////////////////////
//
// Store.h
//
// Copyright 1997, James E. Beveridge.
// Permission is granted to freely reuse this
// code as long as credit is given in the source code.
//
// Contains declarations for a Specialty Store
// object that provides criteria-based creation
// of file converters.
//
///////////////////////////////////////////////////////

//
// Forward references
//
class FileConverterProxyBase;
class FileConverter;

//
// Declare a typedef for the array of proxies.
// This definition uses STL.  For Microsoft
// compilers, you must have Visual C++ 4.2 or later.
// Depending on whose compiler you are using, you
// may need to take out "std::" if your STL
// implementation is not in the std namespace.
//
#if !defined(_MSC_VER) || _MSC_VER >= 1100
    typedef std::vector<const FileConverterProxyBase*>
					ConverterVector;
#elif _MSC_VER > 1010
	// This definition is for Visual C++ 4.2
    typedef vector<
                const FileConverterProxyBase*,
                allocator< const FileConverterProxyBase *>
        > ConverterVector;

	#define bool int
	#define true 1
	#define false 0
#else
    #error This program requires Visual C++ 4.2 or later.
#endif

//////////////////////////////////////////////
//
// FileConverterStore
//

class FileConverterStore
{

public:
    // Construction and destruction
    FileConverterStore();
    ~FileConverterStore();

	// Proxy classes call this to register.
    void Register(const FileConverterProxyBase* proxy);

	// Virtual ctor that creates by a criteria
    FileConverter* CreateByExtension(const char* ext);

	// Together, these provide support for walking
	// all known classes. It would be better to use
	// an STL iterator, but this is simpler for the
	// non-STL literate.
    int GetCount();
    const FileConverterProxyBase* GetAt(int i);

private:

	// Do what the ctor would do normally.
    void Init();

	// Pointer to the array of proxies. This
	// MUST be a pointer. If you try to put
	// the member here directly, your program
	// will fail due to global ctor ordering.
    ConverterVector* m_pConverters;
};

extern FileConverterStore gConverterStore;
