// This source code is a small sub-set of the Objective Grid C++ Library
// from Stingray Software, Inc.

// For more information visit: http://www.stingsoft.com

//  Author:		    Stefan Hoenig
//  Created:		11/14/97
//

// ddjmouse.cpp : implementation of the CDDJIntelliMousePlugin class
//

#include "stdafx.h"
#include "resource.h"

#include "ddjmouse.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

// Steps to add support for intelli mouse are:

// Add the files (ddjmouse.* and ddjplgin.*) to your project 
// and then integrate CDDJIntelliMousePlugin as follows:

// a) add forward declaration in view header file:
// class CDDJPluginComponent;
// and add an attribbute
// CDDJPluginComponent* m_pImousePlugin; 
// to the view class declaration

// b) in the view. cpp file include
// #include "ddjmouse.h"

// In OniInitialUpdate add the following two lines 
//	m_pImousePlugin = new CDDJIntelliMousePlugin();
//	m_pImousePlugin->PlugIn(this);
//
// and set/reset m_pImousePlugin in ctor/dtor as follows

/*
CSampleView::CSampleView()
{
	m_pImousePlugin = NULL;
}

CSampleView::~CSampleView()
{
	delete m_pImousePlugin;
}
*/

// c) add an WindowProc override:
//
/*
LRESULT CSampleView::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
   	LRESULT result;

	if (m_pImousePlugin)
   	{
		m_pImousePlugin->HandleMessage(message, wParam, lParam, &result);

   		if (m_pImousePlugin->m_bExitMesssage)
			return result;
	}
}

	return CView::WindowProc(message, wParam, lParam);
}
*/
 
// d) Copy the resources res\gx*.cur and res\gx*.bmp
// into the res directory of your project and insert the following lines in your .rc file:

/*
/////////////////////////////////////////////////////////////////////////////
//
// Bitmap
//

DDJ_IDB_ORIGIN_ALL       BITMAP  DISCARDABLE     "RES\\GXIMALL.BMP"
DDJ_IDB_ORIGIN_VERT      BITMAP  DISCARDABLE     "RES\\GXIMVERT.BMP"
DDJ_IDB_ORIGIN_HORZ      BITMAP  DISCARDABLE     "RES\\GXIMHORZ.BMP"
/////////////////////////////////////////////////////////////////////////////
//
// Cursor
//

DDJ_IDC_IMU              CURSOR  DISCARDABLE     "RES\\GXIMU.CUR"
DDJ_IDC_IML              CURSOR  DISCARDABLE     "RES\\GXIML.CUR"
DDJ_IDC_IMR              CURSOR  DISCARDABLE     "RES\\GXIMR.CUR"
DDJ_IDC_IMD              CURSOR  DISCARDABLE     "RES\\GXIMD.CUR"
DDJ_IDC_IMA              CURSOR  DISCARDABLE     "RES\\GXIMA.CUR"

and in resource.h

#define DDJ_IDB_ORIGIN_ALL               1130
#define DDJ_IDB_ORIGIN_VERT              1131
#define DDJ_IDC_IMU                     1131
#define DDJ_IDB_ORIGIN_HORZ              1132
#define DDJ_IDC_IML                      1132
#define DDJ_IDC_IMR                      1133
#define DDJ_IDC_IMD                      1134
#define DDJ_IDC_IMA                      1135

*/


// If you are using the window in a dialog, you have
// to add a special override for your parent dialog.
// This will make sure that intelli mouse support
// will also work under Win95 and WinNt 3.51. See
// CAboutDlg::OnRegisteredMouseWheel
// in 1stgrid.cpp.

const int nScrollTimer = 991;

UINT DDJGetMouseScrollLines(BOOL bForceFresh = FALSE);

// Timer id for scrolling 

// determine number of elements in an array (not bytes)
#define _ddjcountof(array) (sizeof(array)/sizeof(array[0]))

const int nOffs = 15;

#define DD_DEFLOCKMOUSETICKS 400  // ClickLock
#define DD_DEFSCROLLSPEED 600   // slowest scroll speed
#define DD_DEFSCROLLACCEL 2   // Interval will be (600/(nPixels/2))
#define DD_DEFSCROLLBYPIX 40	// if more than 40 pixels start scrolling more than one line at a time

// ClickLock locks down the mouse button after a single click. 
// Just click and hold down mouse button for a momemnt
// and your click is locked. With ClickLock you can scroll
// the grid very easy. Click again to release ClickLock. To adjust
// the amount of time you need to hold down the mouse wheel
// button before it locks change the above DD_DEFLOCKMOUSETICKS value.
 
struct DDJ_INTELLI_STATE
{
	int nLockMouseTicks;
	int nScrollSpeed;
	int nScrollAccel;
	int nScrollByPix;
	CString sOriginWndClass;

	HCURSOR m_hcurLast;
	HCURSOR m_hcurDestroy;
	UINT m_idcPrimaryLast;

	DDJ_INTELLI_STATE();
};

DDJ_INTELLI_STATE _ddjIntelliState;

DDJ_INTELLI_STATE::DDJ_INTELLI_STATE()
{
	nLockMouseTicks = DD_DEFLOCKMOUSETICKS;
	nScrollSpeed = DD_DEFSCROLLSPEED;
	nScrollAccel = DD_DEFSCROLLACCEL;
	nScrollByPix = DD_DEFSCROLLBYPIX;
	m_hcurLast = 0;
	m_hcurDestroy = 0; 
	m_idcPrimaryLast = 0; 
}

// DDJGetMouseScrollLines is copied from the MFC CScrollView code. 

UINT DDJGetMouseScrollLines(BOOL bForceFresh /* = FALSE */)
{
	// if we've already got it and we're not refreshing,
	// return what we've already got

	static BOOL bGotScrollLines = FALSE;
	static UINT uCachedScrollLines = 0;

	if (!bForceFresh && bGotScrollLines)
		return uCachedScrollLines;

	// see if we can find the mouse window

	bGotScrollLines = TRUE;

	static UINT msgGetScrollLines = 0;
	static WORD nRegisteredMessage = 0;

	if (nRegisteredMessage == 0)
	{
		msgGetScrollLines = ::RegisterWindowMessage(MSH_SCROLL_LINES);
		if (msgGetScrollLines == 0)
			nRegisteredMessage = 1;     // couldn't register!  never try again
		else
			nRegisteredMessage = 2;     // it worked: use it
	}

	if (nRegisteredMessage == 2)
	{
		HWND hwMouseWheel = NULL;
		hwMouseWheel = FindWindow(MSH_WHEELMODULE_CLASS, MSH_WHEELMODULE_TITLE);
		if (hwMouseWheel && msgGetScrollLines)
		{
			uCachedScrollLines = (UINT)
				::SendMessage(hwMouseWheel, msgGetScrollLines, 0, 0);
			return uCachedScrollLines;
		}
	}

	// couldn't use the window -- try system settings

	OSVERSIONINFO ver;
	memset(&ver, 0, sizeof(ver));
	ver.dwOSVersionInfoSize = sizeof(ver);

	uCachedScrollLines = 3; // reasonable default
	if (!::GetVersionEx(&ver))
		return uCachedScrollLines;

	if ((ver.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS ||
		  ver.dwPlatformId == VER_PLATFORM_WIN32_NT)
		&& ver.dwMajorVersion < 4)
	{
		HKEY hKey;
		if (RegOpenKeyEx(HKEY_CURRENT_USER,  _T("Control Panel\\Desktop"),
				0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
		{
			TCHAR szData[128];
			DWORD dwKeyDataType;
			DWORD dwDataBufSize = _ddjcountof(szData);

			if (RegQueryValueEx(hKey, _T("WheelScrollLines"), NULL, &dwKeyDataType,
					(LPBYTE) &szData, &dwDataBufSize) == ERROR_SUCCESS)
			{
				uCachedScrollLines = _tcstoul(szData, NULL, 10);
			}
			RegCloseKey(hKey);
		}
	}
	else if (ver.dwPlatformId == VER_PLATFORM_WIN32_NT && ver.dwMajorVersion >= 4)
	{
		::SystemParametersInfo(SPI_GETWHEELSCROLLLINES, 0, &uCachedScrollLines, 0);
	}

	return uCachedScrollLines;
}

/////////////////////////////////////////////////////////////////////////////
// CDDJMouseWheelOriginWnd

IMPLEMENT_DYNAMIC(CDDJMouseWheelOriginWnd, CWnd)

CDDJMouseWheelOriginWnd::CDDJMouseWheelOriginWnd()
{
}

BOOL CDDJMouseWheelOriginWnd::CreateWnd(CWnd* pParent, UINT nID)
{
	if (m_bmOrigin.GetSafeHandle() == 0 && !m_bmOrigin.LoadBitmap(nID))
	{
		TRACE0("Failed to load bitmap in CDDJMouseWheelOriginWnd::CreateWnd\n");
		ASSERT(0);
		// ASSERTION-> Please check if the specified resource is available. 
		// If you linked static to Objective Grid - Is gxres.rc included? ->END

		return FALSE;
	}

	// Register Window Class
	if (_ddjIntelliState.sOriginWndClass.IsEmpty())
		_ddjIntelliState.sOriginWndClass = AfxRegisterWndClass(CS_SAVEBITS);

	if (!CreateEx(
			0, _ddjIntelliState.sOriginWndClass, NULL, WS_CLIPSIBLINGS|WS_CHILD,
			0, 0, 1, 1,
			pParent->GetSafeHwnd(), NULL))
	{
		TRACE0("Failed to create window in CDDJMouseWheelOriginWnd::CreateWnd\n");
		ASSERT(0);

		return FALSE;
	}

	return TRUE;
};

BEGIN_MESSAGE_MAP(CDDJMouseWheelOriginWnd, CWnd)
	//{{AFX_MSG_MAP(CDDJMouseWheelOriginWnd)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// DrawTransparentBitmap
//
// This function lets you draw transparent bitmaps. The parameter
// cTransparentColor specifies the color for transparent pixels.
// All pixels in the bitmap which should be transparent should be
// marked with this color.
//
// The function was copied and adapated from knowledge base article Q79212
// Title: Drawing Transparent Bitmaps

void CDDJMouseWheelOriginWnd::DrawTransparentBitmap(CDC* pDC, CBitmap* pBitmap, int xStart,
	int yStart, COLORREF cTransparentColor)
{
   CBitmap    bmAndBack, bmAndObject, bmAndMem, bmSave;
   CDC        dcMem, dcBack, dcObject, dcTemp, dcSave;

   dcTemp.CreateCompatibleDC(pDC);
   dcTemp.SelectObject(pBitmap);   // Select the bitmap

   BITMAP     bm;
   pBitmap->GetObject(sizeof(BITMAP), (LPSTR)&bm);

   CPoint     ptSize;
   ptSize.x = bm.bmWidth;            // Get width of bitmap
   ptSize.y = bm.bmHeight;           // Get height of bitmap
   dcTemp.DPtoLP(&ptSize, 1);      // Convert from device
									 // to logical points

   // Create some DCs to hold temporary data.
   dcBack.CreateCompatibleDC(pDC);
   dcObject.CreateCompatibleDC(pDC);
   dcMem.CreateCompatibleDC(pDC);
   dcSave.CreateCompatibleDC(pDC);

   // Create a bitmap for each DC. DCs are required for a number of
   // GDI functions.

   // Monochrome DC
   bmAndBack.CreateBitmap(ptSize.x, ptSize.y, 1, 1, NULL);

   // Monochrome DC
   bmAndObject.CreateBitmap(ptSize.x, ptSize.y, 1, 1, NULL);

   bmAndMem.CreateCompatibleBitmap(pDC, ptSize.x, ptSize.y);
   bmSave.CreateCompatibleBitmap(pDC, ptSize.x, ptSize.y);

   // Each DC must select a bitmap object to store pixel data.
   CBitmap* pbmBackOld   = dcBack.SelectObject(&bmAndBack);
   CBitmap* pbmObjectOld = dcObject.SelectObject(&bmAndObject);
   CBitmap* pbmMemOld    = dcMem.SelectObject(&bmAndMem);
   CBitmap* pbmSaveOld   = dcSave.SelectObject(&bmSave);

   // Set proper mapping mode.
   dcTemp.SetMapMode(pDC->GetMapMode());

   // Save the bitmap sent here, because it will be overwritten.
   dcSave.BitBlt(0, 0, ptSize.x, ptSize.y, &dcTemp, 0, 0, SRCCOPY);

   // Set the background color of the source DC to the color.
   // contained in the parts of the bitmap that should be transparent
   COLORREF cColor = dcTemp.SetBkColor(cTransparentColor);

   // Create the object mask for the bitmap by performing a BitBlt
   // from the source bitmap to a monochrome bitmap.
   dcObject.BitBlt(0, 0, ptSize.x, ptSize.y, &dcTemp, 0, 0,
		  SRCCOPY);

   // Set the background color of the source DC back to the original
   // color.
   dcTemp.SetBkColor(cColor);

   // Create the inverse of the object mask.
   dcBack.BitBlt(0, 0, ptSize.x, ptSize.y, &dcObject, 0, 0,
		  NOTSRCCOPY);

   // Copy the background of the main DC to the destination.
   dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, pDC, xStart, yStart,
		  SRCCOPY);

   // Mask out the places where the bitmap will be placed.
   dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, &dcObject, 0, 0, SRCAND);

   // Mask out the transparent colored pixels on the bitmap.
   dcTemp.BitBlt(0, 0, ptSize.x, ptSize.y, &dcBack, 0, 0, SRCAND);

   // XOR the bitmap with the background on the destination DC.
   dcMem.BitBlt(0, 0, ptSize.x, ptSize.y, &dcTemp, 0, 0, SRCPAINT);

   // Copy the destination to the screen.
   pDC->BitBlt(xStart, yStart, ptSize.x, ptSize.y, &dcMem, 0, 0,
		  SRCCOPY);

   // Place the original bitmap back into the bitmap sent here.
   dcTemp.BitBlt(0, 0, ptSize.x, ptSize.y, &dcSave, 0, 0, SRCCOPY);

   // Reset the memory bitmaps.
   dcBack.SelectObject(pbmBackOld);
   dcObject.SelectObject(pbmObjectOld);
   dcMem.SelectObject(pbmMemOld);
   dcSave.SelectObject(pbmSaveOld);

   // Memory DCs and Bitmap objects will be deleted automatically
}

void CDDJMouseWheelOriginWnd::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	CRect rect;
	GetClientRect(rect);

	// choose bitmap for the header
	CBitmap* pBitmap = &m_bmOrigin;
	CDC* pDC = &dc;

	if (pBitmap)
	{
		// Bitmap size
		BITMAP     bm;
		pBitmap->GetObject(sizeof(BITMAP), (LPSTR)&bm);

		CPoint     ptSize;
		ptSize.x = bm.bmWidth;            // Get width of bitmap
		ptSize.y = bm.bmHeight;           // Get height of bitmap
		pDC->DPtoLP(&ptSize, 1);      // Convert from device to logical points

		// Draw bitmap
		if (rect.Width() >= ptSize.x && rect.Height() >= ptSize.x)
		{
			// must have at least the first bitmap loaded before calling DrawItem
			ASSERT(pBitmap->m_hObject != NULL);     // required

			int x = rect.left + max(1, (rect.Width()-ptSize.x)/2);
			int y = rect.top + max(1, (rect.Height()-ptSize.y)/2);

			DrawTransparentBitmap(pDC, // The destination DC.
				pBitmap, // The bitmap to be drawn.
				x,        // X coordinate.
				y,        // Y coordinate.
				RGB(255,0,0)); // The color for transparent
								   // pixels (white grey).

		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDDJIntelliMousePlugin

IMPLEMENT_DYNAMIC(CDDJIntelliMousePlugin, CDDJPluginComponent)

BEGIN_MESSAGE_MAP(CDDJIntelliMousePlugin, CDDJPluginComponent)
	//{{AFX_MSG_MAP(CDDJIntelliMousePlugin)
	ON_WM_MOUSEWHEEL()
	ON_WM_MBUTTONDOWN()
	ON_WM_MBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_SETCURSOR()
	ON_WM_CANCELMODE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_MESSAGE(WM_SETTINGCHANGE, OnSettingChange)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDDJIntelliMousePlugin construction/destruction

CDDJIntelliMousePlugin::CDDJIntelliMousePlugin()
{
	m_bMouseWheelDrag = FALSE;
	m_idcCursor = 0;
}

CDDJIntelliMousePlugin::~CDDJIntelliMousePlugin()
{
}

/////////////////////////////////////////////////////////////////////////////
// CDDJIntelliMousePlugin diagnostics

#ifdef _DEBUG
void CDDJIntelliMousePlugin::AssertValid() const
{
	CDDJPluginComponent::AssertValid();
}

void CDDJIntelliMousePlugin::Dump(CDumpContext& dc) const
{
	CDDJPluginComponent::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDDJIntelliMousePlugin message handlers

BOOL CDDJIntelliMousePlugin::OnMouseWheel(UINT nFlags, short zDelta, CPoint point)
{
	// only this plugin should handle WM_MOUSEWHEEL message
	m_bExitMesssage = TRUE;

	// cancel any existing autoscroll behavior
	if (m_bMouseWheelDrag)
		KillMouseWheel(point);	

	if (nFlags == MK_CONTROL)
	{
		int zoom = OnIMouseGetZoom();

		if (zoom == 0)
			return FALSE;	// zooming not supported
		
		if (zDelta > 0 && zoom > 50)
			OnIMouseSetZoom(zoom-10);
		else if (zDelta < 0 && zoom < 200)
			OnIMouseSetZoom(zoom+10);
	}
	else if (nFlags == 0 || nFlags == MK_SHIFT)
	{
		if (zDelta == 0)
			return FALSE;

		DWORD dwStyle = GetStyle();
		CWnd* pWnd = CWnd::FromHandle(m_hWnd);
		CScrollBar* pBar = pWnd->GetScrollBarCtrl(SB_VERT);
		bool bHasVertBar = ((pBar != NULL) && pBar->IsWindowEnabled()) ||
						(dwStyle & WS_VSCROLL);

		pBar = pWnd->GetScrollBarCtrl(SB_HORZ);
		bool bHasHorzBar = ((pBar != NULL) && pBar->IsWindowEnabled()) ||
						(dwStyle & WS_HSCROLL);

		UINT uWheelScrollLines = DDJGetMouseScrollLines();
		int nToScroll = abs(::MulDiv(-zDelta, uWheelScrollLines, WHEEL_DELTA));

		// Scroll horizontal if SHIFT is pressed or no vertical scrollbar
		if (bHasHorzBar && (nFlags == MK_SHIFT || !bHasVertBar))
		{
			if (uWheelScrollLines == WHEEL_PAGESCROLL)
			{
				if (zDelta > 0)
					OnIMouseHScroll(SB_PAGEUP, 1);
				else 
					OnIMouseHScroll(SB_PAGEDOWN, 1);
			}
			else
			{
				if (nToScroll == 0)
					return FALSE;

				if (zDelta > 0)
					OnIMouseHScroll(SB_LINEUP, nToScroll);
				else 
					OnIMouseHScroll(SB_LINEDOWN, nToScroll);
			}
		}
		else // scroll vertically
		{
			if (uWheelScrollLines == WHEEL_PAGESCROLL)
			{
				if (zDelta > 0)
					OnIMouseVScroll(SB_PAGEUP, 1);
				else 
					OnIMouseVScroll(SB_PAGEDOWN, 1);
			}
			else
			{
				if (nToScroll == 0)
					return FALSE;

				if (zDelta > 0)
					OnIMouseVScroll(SB_LINEUP, nToScroll);
				else 
					OnIMouseVScroll(SB_LINEDOWN, nToScroll);
			}
		}
	}
	else
		return FALSE;

	return TRUE;
}

//////////////////
// User changed system settings (possibly Wheel mouse).
// Reload  MouseWheel paramerters.
//
LRESULT CDDJIntelliMousePlugin::OnSettingChange(WPARAM, LPARAM)
{
	DDJGetMouseScrollLines(TRUE);
	return 0;
}

void CDDJIntelliMousePlugin::OnMButtonDown(UINT nFlags, CPoint point) 
{
	BOOL bCtl = GetKeyState(VK_CONTROL) & 0x8000;

	if (m_bMouseWheelDrag)
		KillMouseWheel(point);	
	
	else if (!bCtl && nFlags == MK_MBUTTON)
	{
		SetCapture();

		GetCursorPos(&point);
		point -= CPoint(nOffs,nOffs);
		SetCursorPos(point.x, point.y);
		ScreenToClient(&point);
		
		m_bMouseWheelDrag = TRUE;
		m_ptMouseWheelOrg = point;
		m_nMouseWheelTimer = SetTimer(nScrollTimer, 10, NULL);
		m_dwMouseWheelTick = GetTickCount();
		m_dwMouseWheelLastTick = 0;
		m_nMouseWheelNextTicks = 0;
		m_nMouseWheelLastDirection = -1;

		m_bMouseWheelCanVertScroll = TRUE;
		m_bMouseWheelCanHorzScroll = TRUE;

		int nIdBmp;
		if (m_bMouseWheelCanVertScroll && m_bMouseWheelCanHorzScroll)
			nIdBmp = DDJ_IDB_ORIGIN_ALL;
		else if (m_bMouseWheelCanVertScroll)
			nIdBmp = DDJ_IDB_ORIGIN_VERT;
		else 
			nIdBmp = DDJ_IDB_ORIGIN_HORZ;

		m_pWndOrigin = new CDDJMouseWheelOriginWnd();
		m_pWndOrigin->CreateWnd(this, nIdBmp);
		m_pWndOrigin->MoveWindow(CRect(m_ptMouseWheelOrg.x, m_ptMouseWheelOrg.y, m_ptMouseWheelOrg.x+30, m_ptMouseWheelOrg.y+30));
		m_pWndOrigin->ShowWindow(SW_SHOW);

		m_bExitMesssage = TRUE;
	}
	else
		CDDJPluginComponent::OnMButtonDown(nFlags, point);
}

void CDDJIntelliMousePlugin::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (m_bMouseWheelDrag)
	{
		m_bExitMesssage = TRUE;
		ShowImouseCursor(m_idcCursor);
	}

	CDDJPluginComponent::OnMouseMove(nFlags, point);
}


BOOL CDDJIntelliMousePlugin::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	if (m_bMouseWheelDrag)
	{
		m_bExitMesssage = TRUE;
		return TRUE;    // we will handle it in the mouse move
	}

	return CDDJPluginComponent::OnSetCursor(pWnd, nHitTest, message);
}

void CDDJIntelliMousePlugin::OnMButtonUp(UINT nFlags, CPoint point) 
{
	if (m_dwMouseWheelTick == 0 || GetTickCount()-m_dwMouseWheelTick < (DWORD) _ddjIntelliState.nLockMouseTicks)
		KillMouseWheel(point);

	CDDJPluginComponent::OnMButtonUp(nFlags, point);
}

void CDDJIntelliMousePlugin::OnLButtonDown(UINT nFlags, CPoint point) 
{
	KillMouseWheel(point);	
	
	CDDJPluginComponent::OnLButtonDown(nFlags, point);
}

void CDDJIntelliMousePlugin::OnRButtonDown(UINT nFlags, CPoint point) 
{
	KillMouseWheel(point);	

	CDDJPluginComponent::OnRButtonDown(nFlags, point);
}

void CDDJIntelliMousePlugin::OnCancelMode() 
{
	KillMouseWheel(CPoint(0,0));	

	CDDJPluginComponent::OnCancelMode();
}

void CDDJIntelliMousePlugin::KillMouseWheel(CPoint point) 
{
	if (m_bMouseWheelDrag)
	{
		if (m_pWndOrigin)
			m_pWndOrigin->DestroyWindow();
		delete m_pWndOrigin;
		m_bMouseWheelDrag = FALSE;
		ReleaseCapture();
		KillTimer(m_nMouseWheelTimer);
		SetImouseCursor(NULL);

		GetCursorPos(&point);
		point += CPoint(nOffs,nOffs);
		SetCursorPos(point.x, point.y);
		ScreenToClient(&point);
	}
}

enum Direction
{
	SCROLL_UP               = 1,
	SCROLL_LEFT             = 2,
	SCROLL_DOWN             = 3,
	SCROLL_RIGHT            = 4,
};

void CDDJIntelliMousePlugin::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent == nScrollTimer && m_bMouseWheelDrag)
	{
		CPoint ptCur;
		GetCursorPos(&ptCur);
		ScreenToClient(&ptCur);

		CPoint pt = ptCur - m_ptMouseWheelOrg;

		int direction;
		int nScroll;
		int nAccel;

		if (abs(pt.x) > abs(pt.y) && m_bMouseWheelCanHorzScroll || !m_bMouseWheelCanVertScroll)
		{
			direction = pt.x > 0 ? SCROLL_RIGHT : SCROLL_LEFT;
			nScroll = abs(pt.x) < 8 ? 0 : abs(pt.x) < _ddjIntelliState.nScrollByPix ? 1 : (abs(pt.x)-_ddjIntelliState.nScrollByPix)/_ddjIntelliState.nScrollAccel+1;
			nAccel = max(1, (abs(pt.x)-10)/_ddjIntelliState.nScrollAccel);
		}
		else
		{
			direction = pt.y > 0 ? SCROLL_DOWN : SCROLL_UP;
			nScroll = abs(pt.y) < 8 ? 0 : abs(pt.y) < _ddjIntelliState.nScrollByPix ? 1 : (abs(pt.y)-_ddjIntelliState.nScrollByPix)/_ddjIntelliState.nScrollAccel+1;
			nAccel = max(1, (abs(pt.y)-10)/_ddjIntelliState.nScrollAccel);
		}

		DWORD dwTick = GetTickCount();

		if (nScroll == 0 && abs(pt.x) < 8 && abs(pt.y) < 8)
			direction = -1;
		else
			m_dwMouseWheelTick = 0;

		int nCursor;
		UINT nSBCode = 0;
		BOOL bHorz = 0;
		switch (direction)
		{
		case SCROLL_LEFT: nCursor = DDJ_IDC_IML; bHorz = TRUE; nSBCode = SB_LINEUP; break;
		case SCROLL_RIGHT: nCursor = DDJ_IDC_IMR; bHorz = TRUE; nSBCode = SB_LINEDOWN; break;
		case SCROLL_UP: nCursor = DDJ_IDC_IMU; bHorz = FALSE; nSBCode = SB_LINEUP; break;
		case SCROLL_DOWN: nCursor = DDJ_IDC_IMD; bHorz = FALSE; nSBCode = SB_LINEDOWN; break;
		default: nCursor = DDJ_IDC_IMA; break;
		}

		SetImouseCursor(nCursor);
		m_nMouseWheelLastDirection = direction;

		// scroll if necessary
		if (nScroll > 0 && (direction != m_nMouseWheelLastDirection
			|| dwTick - m_dwMouseWheelLastTick > (DWORD) m_nMouseWheelNextTicks))
		{
			m_dwMouseWheelLastTick = dwTick;
			m_nMouseWheelNextTicks = _ddjIntelliState.nScrollSpeed/nAccel;

			// Hide origin window - old window contents will be restored 
			m_pWndOrigin->DestroyWindow();

			// Scroll
			if (bHorz)
				OnIMouseHScroll(nSBCode, nScroll);
			else
				OnIMouseVScroll(nSBCode, nScroll);

			UpdateWindow();

			// Show origin window
			m_pWndOrigin->CreateWnd(this, 0);
			m_pWndOrigin->MoveWindow(CRect(m_ptMouseWheelOrg.x, m_ptMouseWheelOrg.y, m_ptMouseWheelOrg.x+30, m_ptMouseWheelOrg.y+30));
			m_pWndOrigin->ShowWindow(SW_SHOW);
		}

	}
	else
		CDDJPluginComponent::OnTimer(nIDEvent);
}

void CDDJIntelliMousePlugin::SetImouseCursor(UINT idcCursor)
{
	m_idcCursor = idcCursor;
	ShowImouseCursor(m_idcCursor);
}

BOOL CDDJIntelliMousePlugin::ShowImouseCursor(UINT idcPrimary)
{
	if (idcPrimary != 0)
	{
		HCURSOR hcurToDestroy = NULL;
		if (idcPrimary != _ddjIntelliState.m_idcPrimaryLast)
		{
			HINSTANCE hInst = AfxFindResourceHandle(
				MAKEINTRESOURCE(idcPrimary), RT_GROUP_CURSOR);

			// load in another cursor
			hcurToDestroy = _ddjIntelliState.m_hcurDestroy;

			// Note: If this LoadCursor call fails, it is likely that
			// gxres.rc file is not included in your resource file.
			// To correct the situation, include gxres.rc into your
			// resource script.
			// This should be done using the Resource.Set Includes... command.
			// and including gxresrc.h and gxres.rc

			if ((_ddjIntelliState.m_hcurDestroy = _ddjIntelliState.m_hcurLast =
			   ::LoadCursor(hInst, MAKEINTRESOURCE(idcPrimary))) == NULL)
			{
				// will not look as good
				TRACE0("Warning: Could not find cursor\n");
			}
			_ddjIntelliState.m_idcPrimaryLast = idcPrimary;
		}
		ASSERT(_ddjIntelliState.m_hcurLast != NULL);
		SetCursor(_ddjIntelliState.m_hcurLast);
		ASSERT(_ddjIntelliState.m_hcurLast != hcurToDestroy);
		if (hcurToDestroy != NULL)
			DestroyCursor(hcurToDestroy); // destroy after being set

		return TRUE;
	}

	return FALSE;
}

/////////////////////////////////////////////////////////////////////////////////////

// Let's send a window messages as default but you might
// also subclass CDDJIntelliMousePlugin and override SetZoom,
// GetZoom etc.

int CDDJIntelliMousePlugin::OnIMouseGetZoom()
{
	return (int) SendMessage(WM_DDJ_IMOUSE_GETZOOM, 0, 0);
}

void CDDJIntelliMousePlugin::OnIMouseSetZoom(int nZoom)
{
	SendMessage(WM_DDJ_IMOUSE_SETZOOM, (WPARAM) nZoom, 0);
}

void CDDJIntelliMousePlugin::OnIMouseVScroll(UINT nSBCode, UINT nScroll)
{
	// Call the virtual GetScrollBarCtrl method for the
	// original window
	CWnd* pWnd = CWnd::FromHandle(m_hWnd);
	CScrollBar* pBar = pWnd->GetScrollBarCtrl(SB_VERT);

	CSplitterWnd* pSplitter = (CSplitterWnd*) GetParent();
	if (pBar && pSplitter->IsKindOf(RUNTIME_CLASS(CSplitterWnd)))
		pWnd = pSplitter;

	while (nScroll-- > 0)
		pWnd->SendMessage(WM_VSCROLL, MAKEWPARAM(nSBCode, 0), (LPARAM) pBar->GetSafeHwnd());
}

void CDDJIntelliMousePlugin::OnIMouseHScroll(UINT nSBCode, UINT nScroll)
{
	// Call the virtual GetScrollBarCtrl method for the
	// original window
	CWnd* pWnd = CWnd::FromHandle(m_hWnd);
	CScrollBar* pBar = pWnd->GetScrollBarCtrl(SB_HORZ);

	CSplitterWnd* pSplitter = (CSplitterWnd*) GetParent();
	if (pBar && pSplitter->IsKindOf(RUNTIME_CLASS(CSplitterWnd)))
		pWnd = pSplitter;

	while (nScroll-- > 0)
		pWnd->SendMessage(WM_HSCROLL, MAKEWPARAM(nSBCode, 0), (LPARAM) pBar->GetSafeHwnd());
}


