/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
dispatcher.h   CDispatcher class definition
               This is the interface between a client application
                 and the notifier dispatcher.  

               The client subclasses this interface to provide a 
                 DispatchToOne function which is aware of the specific
                 notifier classes used by the client.
---------------------------------------------------------------------------*/
#ifndef DISPATCHER_H
#define DISPATCHER_H

#ifndef NOTIFIER_EXPORT
#if defined ( NOTIFY_DLL )
   #define NOTIFIER_EXPORT _declspec(dllexport)
#else
   #define NOTIFIER_EXPORT 
#endif
#endif

#include "subscriber.h"
class CNotifier;
//---------------------------------------------------------------------------
class NOTIFIER_EXPORT CDispatcher
//---------------------------------------------------------------------------
{
public:        // Constructors and destructors
   CDispatcher(void);
   virtual ~CDispatcher(void);
   HRESULT Init(UINT nTimerPeriod=50); // dispatch notifiers every 50 ms

   // Notifier post and send
      // Post: queue a notifier for subsequent dispatch
   HRESULT Post(CNotifier *pNotifier);
      // Send: immediately dispatch a notifier
   HRESULT Send(CNotifier *pNotifier, 
                CNotifier::AUTODELETE bDelete);

   // Subscribe and Unsubscribe
      // IsSubscribed: return true if a subscriber will be sent this notifier 
   bool IsSubscribed(CSubscriber *pSubscriber, 
                     UINT nSubscription);
      // Subscribe: add subscriber to the list that receives this notifier
   HRESULT Subscribe(CSubscriber *pSubscriber, 
                     UINT nSubscription);
      // Unsubscribe: no longer send this notifier type to the subscriber
   HRESULT Unsubscribe(CSubscriber *pSubscriber, 
                       UINT nSubscription);
      // UnsubscribeAll: send no notifiers to this subscriber
   HRESULT UnsubscribeAll(CSubscriber *pSubscriber);
      // SetSubscriberPriority:  move this subscriber ahead or behind other
      //                         subscribers in the list
   HRESULT SetSubscriberPriority(CSubscriber *pSubscriber,
                                 const CSubscriber::SUBSCRIBER_PRIORITY nPriority);
protected:
      // The client app customizes this function to dispatch to all the
      //    client's custom notifier classes
   virtual HRESULT DispatchToOne(CNotifier *pNotifier, 
                                 CSubscriber *pSubscriber) = 0;
private:
   UINT  m_nTimer;   // timer used to trigger periodic dispatching

   friend class CDispPrivate;
};

#endif // DISPATCHER_H