/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
notifier.h   CNotifier class definition
             This is the base class for all notifiers.  See example
             programs and "Simplifying C++" article for how to 
             create a notifier subclass.
---------------------------------------------------------------------------*/
#ifndef NOTIFY_H
#define NOTIFY_H

#ifndef NOTIFIER_EXPORT
#if defined ( NOTIFY_DLL )
   #define NOTIFIER_EXPORT _declspec(dllexport)
#else
   #define NOTIFIER_EXPORT 
#endif
#endif

class CDispatcher;

//---------------------------------------------------------------------------
class NOTIFIER_EXPORT CNotifier
//---------------------------------------------------------------------------
{
public:
   enum PRIORITY 
      {
      PRIORITY_HIGH,
      PRIORITY_NORMAL,
      PRIORITY_LOW
      };

   enum AUTODELETE
      {
      NO_AUTODELETE,
      YES_AUTODELETE
      };

public:     // Constructors and destructors
   CNotifier(int nClass = 0);
   CNotifier(const CNotifier &src);
   virtual ~CNotifier(void);
   CNotifier &operator=(const CNotifier &event);

   // Operations
      // Post: Queue this notifier for subsequent dispatch
   HRESULT Post(PRIORITY nPriority = PRIORITY_NORMAL);
      // Send: Dispatch this notifier immediately. 
      //       By default, delete the notifier after sending
   HRESULT Send(AUTODELETE bDelete = YES_AUTODELETE);

            // Attributes
      // GetClass: return notifier subclass
   int  GetClass(void)                       { return m_nClass; }
      // SetClass: set notifier subclass
   void SetClass(const int nClass)           { m_nClass = nClass; }

      // GetPriority: return priority that determines order of dispatching
   PRIORITY GetPriority(void)                 { return m_nPriority; }
      // SetPriority: change the sort order of a notifier (before it's queued)
   void     SetPriority(const PRIORITY nPri)  { m_nPriority = nPri; }

protected:  // Implementation
      // SetDispatcher: Called by CDispatcher to set the static pointer to itself
   static void SetDispatcher(CDispatcher *pDispatcher);
      // GetDispatcher: Get pointer to dispatcher
   static CDispatcher * GetDispatcher(void);

protected:  // Data members
   static CDispatcher *m_pDispatcher;    // see SetDispatcher

   int         m_nClass;         // subclass determines which On function is called in CSubscriber
   PRIORITY    m_nPriority;      // determines dispatch order of posted notifiers

   friend class CDispatcher;
}; // end of PTNotifer class definition

#endif // NOTIFY_H