/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

 Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
simcust.h  Customized error simulation for the notifier DLL
---------------------------------------------------------------------------*/
#ifndef SIMERR_NOTIFY_H
#define SIMERR_NOTIFY_H

#include "simdef.h"  // data definitions used in the function prototypes
#include "simerr.h"  // error simulation functions

#ifndef NOTIFIER_EXPORT
#if defined ( NOTIFY_DLL )
   #define NOTIFIER_EXPORT _declspec(dllexport)
#else
   #define NOTIFIER_EXPORT 
#endif
#endif

// These are the different types of errors that you can test.
// If you change this enum, please change the g_errorInfo
// static table in simerr_notify.cpp.
enum ERROR_TYPE
{  // IMPORTANT: The first two entries must always be 
   //            SIMERR_RESULT_ERROR and SIMERR_MEMORY_ERROR
   SIMERR_RESULT_ERROR,        // failing function returns an error code
   SIMERR_MEMORY_ERROR,        // failed allocation returns zero
   SIMERR_MEMORY_EXCEPTION,    // MFC memory exception
   SIMERR_RESOURCE_EXCEPTION,  // MFC resource exception
   SIMERR_FILE_EXCEPTION,      // MFC file exception
   NUM_SIMERR_TYPES
};

//==================== DEFINE IF SIMULATING ERRORS ====================
#ifdef SIMERR 

NOTIFIER_EXPORT bool SimErrCustomInit(int nSeed, SIMERR_TEST_TYPE testType); 

/*---------------------------------------------------------------------------
SimErrTimer  Simulate errors in functions that return zero on failure
---------------------------------------------------------------------------*/
inline UINT
SimErrTimer(const char *strFile, int nLine, UINT nTimer, 
             long nCount = 1, double nProb = 1.0)
{
   if ((nTimer != 0) && 
       SimErr(strFile, nLine, SIMERR_RESULT_ERROR, nCount, nProb)) 
   { 
      KillTimer(0, nTimer);
      nTimer = 0; 
   } 
   return nTimer;
}
/*---------------------------------------------------------------------------
SimErrCreateWin  Simulate errors in functions that create windows
---------------------------------------------------------------------------*/
inline BOOL
SimErrCreateWin(const char *strFile, int nLine, BOOL bCreated, CWnd *pWnd,
             long nCount = 1, double nProb = 1.0)
{
   if ((bCreated) && 
       SimErr(strFile, nLine, SIMERR_RESULT_ERROR, nCount, nProb)) 
   { 
      pWnd->DestroyWindow();
      bCreated = false;
   } 
   return bCreated;
}


//================= DEFINE IF NOT SIMULATING ERRORS ====================
#else
#pragma warning( disable : 4100 )   

inline bool SimErrCustomInit(int , SIMERR_TEST_TYPE )
{ return true; }

inline UINT
SimErrTimer(const char *strFile, int nLine, UINT nTimer, 
             long nCount = 1, double nProb = 1.0)
{
   return nTimer;
}

inline BOOL
SimErrCreateWin(const char *strFile, int nLine, BOOL bCreated, CWnd *pWnd,
                long nCount = 1, double nProb = 1.0)
{
   return bCreated;
}

#pragma warning( default : 4100 )   
#endif // SIMERR

#endif // SIMERR_NOTIFY_H

