/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
subscriber.h   CSubscriber Mixin Class
               Base class for objects that receive CNotifiers.  
               This class is intended for multiple inheritance, so the
               names are chosen to remain unique.
---------------------------------------------------------------------------*/
#ifndef SUBSCRIBER_H
#define SUBSCRIBER_H

#ifndef NOTIFIER_EXPORT
#if defined ( NOTIFY_DLL )
   #define NOTIFIER_EXPORT _declspec(dllexport)
#else
   #define NOTIFIER_EXPORT 
#endif
#endif

#include "notifier.h"

class CDispatcher;
class CNotifier;
//---------------------------------------------------------------------------
class NOTIFIER_EXPORT CSubscriber
{
public:
   enum SUBSCRIBER_PRIORITY
   {
      PRIORITY_HIGH,
      PRIORITY_NORMAL,
      PRIORITY_LOW
   };

public:     // Constructors and destructors
   CSubscriber(SUBSCRIBER_PRIORITY nPriority = PRIORITY_NORMAL);
   CSubscriber(const CSubscriber &src);
   virtual ~CSubscriber(void);
   CSubscriber &operator=(const CSubscriber &src);

   // Accessors
      // GetSubscriberPriority: return relative priority of subscriber
   SUBSCRIBER_PRIORITY GetSubscriberPriority(void);
      // SetSubscriberPriority:  move this subscriber ahead or behind other
      //                         subscribers in the list
   void SetSubscriberPriority(const SUBSCRIBER_PRIORITY nPriority);

      // IsSubscribed: return true if a subscriber will be sent this notifier 
   bool IsSubscribed(UINT nSubscription);
      // Subscribe: add subscriber to the list that receives this notifier
   HRESULT Subscribe(UINT nSubscription);
      // Unsubscribe: no longer send this notifier type to the subscriber
   HRESULT Unsubscribe(UINT nSubscription);

protected:  // Implementation
      // SetDispatcher: Called by CDispatcher to set the static pointer to itself
   static void SetDispatcher(CDispatcher *pDispatcher);
      // GetDispatcher: Get pointer to dispatcher
   static CDispatcher * GetDispatcher(void);
   
protected:  // Data members
   static CDispatcher *m_pDispatcher;           // see SetDispatcher

   SUBSCRIBER_PRIORITY m_nSubscriberPriority;   // determines dispatch order of posted notifiers

   friend class CDispatcher;
}; // end of CSubscriber class definition

#endif //SUBSCRIBER_H