/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
notifier.cpp   CNotifier class implementation
               This is the base class for all notifiers.  See example
               programs and "Simplifying C++" article for how to 
               create a notifier subclass.
---------------------------------------------------------------------------*/
#include "stdafx.h"
#include "notifier.h"
#include "dispatcher.h"
#include "simerr_notify.h"       // error simulation

/******************
*** Static Data ***
******************/
NOTIFIER_EXPORT CDispatcher *CNotifier::m_pDispatcher = 0;

/**********************************
*** Construction and Assignment ***
**********************************/

/*---------------------------------------------------------------------------
default constructor
---------------------------------------------------------------------------*/
CNotifier::CNotifier(int nClass)
{
   m_nClass    = nClass;  
   m_nPriority = PRIORITY_NORMAL;
}
/*---------------------------------------------------------------------------
copy constructor
---------------------------------------------------------------------------*/
CNotifier::CNotifier(const CNotifier &src)
{
   *this = src;
}
/*---------------------------------------------------------------------------
destructor
---------------------------------------------------------------------------*/
CNotifier::~CNotifier(void)
{
}
/*---------------------------------------------------------------------------
Assignment operator
---------------------------------------------------------------------------*/
CNotifier &CNotifier::operator=(const CNotifier &src)
{
   m_nClass    = src.m_nClass;  
   m_nPriority = src.m_nPriority;
   return *this;
}

/*****************
*** Operations ***
*****************/

/*---------------------------------------------------------------------------
Post           Add a notifier to the queue for subsequent dispatching.
---------------------------------------------------------------------------*/
HRESULT CNotifier::Post(PRIORITY nPriority)
{
   HRESULT hr = S_OK;
   if (m_pDispatcher == 0)
   {
      hr = E_FAIL;
   }
   else 
   {
      SetPriority(nPriority);
      hr = m_pDispatcher->Post(this);
      hr = SimErrResult(SIMLINE, hr, E_OUTOFMEMORY);
   }
   return hr;
}

/*---------------------------------------------------------------------------
Send           send a notifier immediately to all subscribers.
---------------------------------------------------------------------------*/
HRESULT CNotifier::Send(AUTODELETE bDelete)
{
   HRESULT hr = S_OK;
   if (m_pDispatcher == 0)
   {
      hr = E_FAIL;
   }
   else 
   {
      hr = m_pDispatcher->Send(this, bDelete);
      hr = SimErrResult(SIMLINE, hr, E_OUTOFMEMORY);
   }
   return hr;
}

/*---------------------------------------------------------------------------
SetDispatcher
---------------------------------------------------------------------------*/
void CNotifier::SetDispatcher(CDispatcher *pDispatcher)
{ 
   m_pDispatcher = pDispatcher; 
}
/*---------------------------------------------------------------------------
GetDispatcher
---------------------------------------------------------------------------*/
CDispatcher * CNotifier::GetDispatcher(void)
{ 
   return m_pDispatcher; 
}

