/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
subscribe.cpp   CSubscriber Mixin Class
---------------------------------------------------------------------------*/
#include "stdafx.h"
#include "subscriber.h"
#include "dispatcher.h"
#include "simerr_notify.h"       // error simulation

/******************
*** Static Data ***
******************/
NOTIFIER_EXPORT CDispatcher *CSubscriber::m_pDispatcher = 0;

/*---------------------------------------------------------------------------
default contructor     
---------------------------------------------------------------------------*/
CSubscriber::CSubscriber(SUBSCRIBER_PRIORITY nPriority)
{
   m_nSubscriberPriority = nPriority;
}
/*---------------------------------------------------------------------------
copy constructor
---------------------------------------------------------------------------*/
CSubscriber::CSubscriber(const CSubscriber &src)
{
   *this = src;
}
/*---------------------------------------------------------------------------
destructor     
---------------------------------------------------------------------------*/
CSubscriber::~CSubscriber(void)
{
   if (m_pDispatcher != 0)
   {
      m_pDispatcher->UnsubscribeAll(this);
   }
}
/*---------------------------------------------------------------------------
assignment operator
---------------------------------------------------------------------------*/
CSubscriber &CSubscriber::operator=(const CSubscriber &src)
{
   m_nSubscriberPriority = src.m_nSubscriberPriority;  
   return *this;
}

/*---------------------------------------------------------------------------
SetDispatcher
---------------------------------------------------------------------------*/
void CSubscriber::SetDispatcher(CDispatcher *pDispatcher)
{ 
   m_pDispatcher = pDispatcher; 
}
/*---------------------------------------------------------------------------
GetDispatcher
---------------------------------------------------------------------------*/
CDispatcher * CSubscriber::GetDispatcher(void)
{ 
   return m_pDispatcher; 
}

/*---------------------------------------------------------------------------
GetSubscriberPriority
---------------------------------------------------------------------------*/
CSubscriber::SUBSCRIBER_PRIORITY CSubscriber::GetSubscriberPriority(void)
{ 
   return m_nSubscriberPriority; 
}
/*---------------------------------------------------------------------------
SetSubscriberPriority
---------------------------------------------------------------------------*/
void CSubscriber::SetSubscriberPriority(const SUBSCRIBER_PRIORITY nPriority)
{ 
   HRESULT hr = E_FAIL;
   if (m_pDispatcher)
   {
      hr = m_pDispatcher->SetSubscriberPriority(this, nPriority);
      if (SUCCEEDED(hr))
      {
         m_nSubscriberPriority = nPriority;
      }
   }
}

/*---------------------------------------------------------------------------
IsSubscribed
---------------------------------------------------------------------------*/
bool CSubscriber::IsSubscribed(UINT nSubscription)
{
   bool bIsSubscribed = false;

   if (m_pDispatcher)
   {
      bIsSubscribed = m_pDispatcher->IsSubscribed(this, nSubscription);
   }
   return bIsSubscribed;
}
/*---------------------------------------------------------------------------
Subscribe      Add a subscription for this subscriber.  
---------------------------------------------------------------------------*/
HRESULT CSubscriber::Subscribe(UINT nSubscription)
{
   HRESULT hr = E_FAIL;

   if (m_pDispatcher)
   {
      hr = m_pDispatcher->Subscribe(this, nSubscription);
      hr = SimErrResult(SIMLINE, hr, E_OUTOFMEMORY);
   }
   return hr;
}

/*---------------------------------------------------------------------------
Unsubscribe    Remove a subscription for this subscriber.  
---------------------------------------------------------------------------*/
HRESULT CSubscriber::Unsubscribe(UINT nSubscription)
{
   HRESULT hr = E_FAIL;

   if (m_pDispatcher)
   {
      hr = m_pDispatcher->Unsubscribe(this, nSubscription);
      hr = SimErrResult(SIMLINE, hr, E_OUTOFMEMORY);
   }
   return hr;
}

