/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
subscriber_element.h   A CSubscriberElement is the object stored in the
                       dispatcher's list of subscribers.
                       This class is used like a data structure, with a
                       few convenient functions.  It's only used by the
                       CDispPrivate class, so we're not hiding the data.
---------------------------------------------------------------------------*/
#ifndef SUBSCRIBER_ELEMENT_H
#define SUBSCRIBER_ELEMENT_H

#include "subscriber.h"
#include "subscriber_mask.h"

class CNotifier;

//---------------------------------------------------------------------------
class CSubscriberElement  
{
public:  // member data
   CSubscriber *     m_pSubscriber;
   CSubscriberMask   m_mask;
   CSubscriber::SUBSCRIBER_PRIORITY m_nPriority;

public:   // member functions

   // constructor, destructor, copy constructor, assignment operator
   CSubscriberElement(CSubscriber *pSubscriber=0);
   virtual ~CSubscriberElement(void);
   CSubscriberElement(const CSubscriberElement &src);
   CSubscriberElement &operator=(const CSubscriberElement &src);

   // operations
      // IsSubscribed: return true if a given bit is set in the mask 
      //               note that bitIndex 3 is 0x00001000.
   bool IsSubscribed(UINT bitIndex) const;
      // IsMaskClear:  return true if all mask bits are zero
   bool IsMaskClear(void) const;
#ifdef _DEBUG
   char *DumpMask(char *pString, int len) const;
#endif

};

/***********************
*** Inline Functions ***
***********************/
/*---------------------------------------------------------------------------
Default Constructor  element is not valid if CSubscriber is zero
---------------------------------------------------------------------------*/
inline CSubscriberElement::CSubscriberElement(CSubscriber *pSubscriber)
{
   m_pSubscriber = pSubscriber;
   m_mask.Clear();
}
/*---------------------------------------------------------------------------
Copy Constructor     
---------------------------------------------------------------------------*/
inline CSubscriberElement::CSubscriberElement(const CSubscriberElement &src) 
{ 
   *this = src; 
}
/*---------------------------------------------------------------------------
Destructor     
---------------------------------------------------------------------------*/
inline CSubscriberElement::~CSubscriberElement(void) 
{
}
/*---------------------------------------------------------------------------
Assignment operator     
---------------------------------------------------------------------------*/
inline CSubscriberElement &CSubscriberElement::operator=(const CSubscriberElement &src)
{
   m_pSubscriber = src.m_pSubscriber;  
   m_mask        = src.m_mask;
   return *this;
}
/*---------------------------------------------------------------------------
IsSubscribed   return true if a given bit is set in the mask 
               note that bitIndex 3 is 0x00001000.
---------------------------------------------------------------------------*/
inline bool CSubscriberElement::IsSubscribed(UINT bitIndex) const
{
   return m_mask.IsSubscribed(bitIndex);
}
/*---------------------------------------------------------------------------
IsMaskClear    return true if all mask bits are zero
---------------------------------------------------------------------------*/
inline bool CSubscriberElement::IsMaskClear(void) const
{
   return m_mask.IsClear();
}

#ifdef _DEBUG
/*---------------------------------------------------------------------------
DumpMask       dump the subscription bit mask.
               Dump functions require that ERRLOG is #defined and the
               utilities DLL is linked, so debug data can be written to
               the circular log file.
---------------------------------------------------------------------------*/
inline char *CSubscriberElement::DumpMask(char *pString, int len) const
{
   return m_mask.DumpMask(pString, len);
}
#endif

#endif // SUBSCRIBER_ELEMENT_H
