/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
subscriberMask.cpp   CSubscriberMask implementation
---------------------------------------------------------------------------*/
#include "stdafx.h"
#include "subscriber_mask.h"

ULONG g_zeroMask = 0;  // this is used if the caller passes an invalid
                       // bit index

/*---------------------------------------------------------------------------
Clear - set the mask to zero 
---------------------------------------------------------------------------*/
void CSubscriberMask::Clear(void)
{
   memset(m_nMask, 0, LEN_SUBSCRIBER_MASK * sizeof(ULONG));
}
/*---------------------------------------------------------------------------
IsClear - return true if all bits are off
---------------------------------------------------------------------------*/
bool CSubscriberMask::IsClear(void) const
{
   bool isClear = true;
   for (int i=0; i < LEN_SUBSCRIBER_MASK; ++i)
   {
      if (m_nMask[i] != 0)
      {
         isClear = false;
         break;
      }
   }
   return isClear;
}
/*---------------------------------------------------------------------------
Subscribe   Given a notifier type, set the corresponding mask bit
---------------------------------------------------------------------------*/
void CSubscriberMask::Subscribe(UINT nBitIndex)
{
   ULONG  nBitMask;
   ULONG &nMaskWord = const_cast<ULONG &> (GetMaskWord(nBitIndex, nBitMask));

   nMaskWord |= nBitMask;
}
/*---------------------------------------------------------------------------
Unsubscribe   Given a notifier type, clear the corresponding mask bit
---------------------------------------------------------------------------*/
void CSubscriberMask::Unsubscribe(UINT nBitIndex)
{
   ULONG  nBitMask;
   ULONG &nMaskWord = const_cast<ULONG &> (GetMaskWord(nBitIndex, nBitMask));

   nMaskWord &= ~nBitMask;
}
/*---------------------------------------------------------------------------
IsSubscribed   Given a notifier type, is the corresponding mask bit on?
---------------------------------------------------------------------------*/
bool CSubscriberMask::IsSubscribed(UINT nBitIndex) const
{
   ULONG        nBitMask;
   const ULONG &nMaskWord     = GetMaskWord(nBitIndex, nBitMask);
   bool         bIsSubscribed = (nMaskWord & nBitMask) != 0;

   return bIsSubscribed;
}
/*---------------------------------------------------------------------------
== comparison operator
---------------------------------------------------------------------------*/
bool operator==(const CSubscriberMask &a,  const CSubscriberMask &b)
{
   bool isEqual = true;
   for (int i=0; i < CSubscriberMask::LEN_SUBSCRIBER_MASK; ++i)
   {
      if (a.m_nMask[i] != b.m_nMask[i])
      {
         isEqual = false;
         break;
      }
   }
   return isEqual;
}
/*---------------------------------------------------------------------------
!= comparison operator
---------------------------------------------------------------------------*/
bool operator!=(const CSubscriberMask &a,  const CSubscriberMask &b)
{
   return !(a==b);
}

/*---------------------------------------------------------------------------
GetMaskWord   Given a notifier type, get the mask and the mask word.

NOTE: THIS FUNCTION ASSUMES THE MASK ARRAY ELEMENTS ARE 4-BYTES LONG.
---------------------------------------------------------------------------*/
const ULONG &CSubscriberMask::GetMaskWord(UINT nBitIndex,      // bit within multi-word mask
                                          ULONG &nMask) const  // output parameter
{
   int  nElementIndex = nBitIndex >> 5;               // divide by # bits in ULONG
   int  nBitInWord    = nBitIndex - (nElementIndex << 5);
   nMask  = 1 << nBitInWord;

   // Validate element index
   // Debug configuration asserts correct index
   // Release configuration returns a clear bit mask
   ASSERT(nElementIndex < LEN_SUBSCRIBER_MASK);
   if (nElementIndex >= LEN_SUBSCRIBER_MASK)
   {
      g_zeroMask = 0;
      return g_zeroMask;
   }

   return m_nMask[nElementIndex];
}

#ifdef _DEBUG
/*---------------------------------------------------------------------------
DumpMask       dump the hex bytes of the mask
               Dump functions require that ERRLOG is #defined and the
               utilities DLL is linked, so debug data can be written to
               the circular log file.
---------------------------------------------------------------------------*/
char *CSubscriberMask::DumpMask(char *pString, int len) const
{
   static char *s_warning = "STRING TOO SHORT";
   char *pCurr = pString;
   if (len < (LEN_SUBSCRIBER_MASK * 10))
   {
      pString = s_warning;
   }
   else
   {
      *pCurr = 0;
      for (int i=0; i < LEN_SUBSCRIBER_MASK; ++i)
      {
         if (i > 0)
         {  // insert space between bytes
            *pCurr++ = ' ';
            *pCurr++ = 0;
         }
         sprintf(pCurr, "%08X", m_nMask[i]);
         pCurr += strlen(pCurr);
      }
   }
   return pString;
}
#endif