/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
subscriber_mask.h   CSubscriberMask class definition

A subscription to a notifier means that a bit is set in a subscriber's
bit mask, at the position reserved for that notifier.  The CNotifierMask
class encapsulates the manipulation of the subscriber mask.

This is important since there can be more than 32 notifier types, meaning
that a mask may extend beyond a simple DWORD.

---------------
IMPORTANT NOTE:
---------------
MAX_NOTIFIER_TYPES is constant.  To increase the capacity of notifier types,
this constant must be changed in the source code and the DLL rebuilt.
---------------------------------------------------------------------------*/
#ifndef SUBSCRIBER_MASK_H
#define SUBSCRIBER_MASK_H

//---------------------------------------------------------------------------
class CSubscriberMask
//---------------------------------------------------------------------------
{
public:
         // number of bits required for the bit mask
   enum { MAX_NOTIFIER_TYPES = 64 };   // this is an arbitrary value for testing
         // Number of 4-byte ULONGs required to contain the mask
   enum { LEN_SUBSCRIBER_MASK = ((((MAX_NOTIFIER_TYPES + 7) / 8) + 3) / 4) };

   // Constructors and destructors
   CSubscriberMask(void);
   CSubscriberMask(const CSubscriberMask &src);
  ~CSubscriberMask(void);
   CSubscriberMask &operator=(const CSubscriberMask &src);

   // Operations
      // Clear:         zero the bit mask
   void Clear(void);
      // IsClear:       return true if the bit mask is zero
   bool IsClear(void) const;
      // Subscribe:     set a bit in the bit mask.  e.g. bit 3 is 0x00001000
   void Subscribe(UINT bitIndex);
      // Unsubscribe:   clear a bit in the bit mask.  
   void Unsubscribe(UINT bitIndex);
      // IsSubscribed:  return true if the corresponding bit is set. 
   bool IsSubscribed(UINT bitIndex) const;

#ifdef _DEBUG
   char *DumpMask(char *pString, int len) const;
#endif

   // Operators
   friend bool operator==(const CSubscriberMask &a,
                          const CSubscriberMask &b);
   friend bool operator!=(const CSubscriberMask &a,
                          const CSubscriberMask &b);

private:
   const ULONG &GetMaskWord(UINT nBitIndex,     
                            ULONG &nMask) const;

   // 32-bits of mask per array entry
   ULONG m_nMask[LEN_SUBSCRIBER_MASK];
};
/***********************
*** Inline Functions ***
***********************/

/*---------------------------------------------------------------------------
default contructor     
---------------------------------------------------------------------------*/
inline CSubscriberMask::CSubscriberMask(void)
{
   Clear();
}
/*---------------------------------------------------------------------------
copy constructor
---------------------------------------------------------------------------*/
inline CSubscriberMask::CSubscriberMask(const CSubscriberMask &src)
{
   *this = src;
}
/*---------------------------------------------------------------------------
destructor     
---------------------------------------------------------------------------*/
inline CSubscriberMask::~CSubscriberMask(void)
{
}
/*---------------------------------------------------------------------------
assignment operator
---------------------------------------------------------------------------*/
inline CSubscriberMask &CSubscriberMask::operator=(const CSubscriberMask &src)
{
   memcpy(m_nMask, src.m_nMask, LEN_SUBSCRIBER_MASK * sizeof(ULONG));
   return *this;
}

#endif //SUBSCRIBER_MASK_H