/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

 Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
BtnUnits.cpp    Button showing temperature units, F for Fahrenheit or
                C for Celsius.  When the user presses the button, 
                the units are toggled and a notifier is sent out. 
---------------------------------------------------------------------------*/
#include "stdafx.h"
#include "thermdef.h"
#include "BtnUnits.h"
#include "unitsNotifier.h"
#include "requestNotifier.h"
#include "simerr_notify.h"       // error simulation


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/**************************************
*** Initialization and Message Maps ***
**************************************/
BEGIN_MESSAGE_MAP(CBtnUnits, CButton)
	//{{AFX_MSG_MAP(CBtnUnits)
	ON_CONTROL_REFLECT(BN_CLICKED, OnClicked)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/*---------------------------------------------------------------------------
CBtnUnits    constructor
---------------------------------------------------------------------------*/
CBtnUnits::CBtnUnits()
          :CButton(), CThermSubscriber()
{
   m_units = CELSIUS;
}

/*---------------------------------------------------------------------------
CBtnUnits    destructor
---------------------------------------------------------------------------*/
CBtnUnits::~CBtnUnits()
{
}

/*---------------------------------------------------------------------------
Init           create the static text control
---------------------------------------------------------------------------*/
HRESULT CBtnUnits::Init(CWnd *pParent, UINT nID)
{
   BOOL bCreated;
   bCreated = Create("C",
                     BS_PUSHBUTTON | WS_CHILD | WS_VISIBLE,
                     m_rectWindow, pParent, nID);
   bCreated = SimErrCreateWin(SIMLINE, bCreated, this);
   if (bCreated)
   {
      m_font.CreateStockObject(ANSI_VAR_FONT);
      SetFont(&m_font, FALSE);
      Subscribe(REQUEST_NOTIFIER);
   }
   return (bCreated) ? S_OK : E_OUTOFMEMORY;
}


/*---------------------------------------------------------------------------
GetDesiredSize increase given width and height as necessary to satisfy 
               minimum size requirements
---------------------------------------------------------------------------*/
void CBtnUnits::GetDesiredSize(int &width, int &height)
{
   HRESULT hr = S_OK;
   CSize   minSize;
	CDC     dc;	
   CFont   font;

   minSize.cx = 100;
   minSize.cy = 24;
   font.CreateStockObject(ANSI_VAR_FONT);
	if (dc.CreateIC("DISPLAY", 0,0,0))
   {
      // First get size of the text
      dc.SelectObject(&font);
      minSize = dc.GetTextExtent("F", 1);
      // Then add a border for the button
      minSize += CSize(24, 6);
   }

   width = max(width, minSize.cx);
   height = minSize.cy;
}

/*---------------------------------------------------------------------------
SetRect        Set the window rectangle
---------------------------------------------------------------------------*/
void CBtnUnits::SetRect(CRect &rect)
{
   if (GetSafeHwnd() && (m_rectWindow != rect))
   {
      MoveWindow(rect, TRUE);
   }
   m_rectWindow = rect;
}


/*---------------------------------------------------------------------------
OnRequestNotifier    someone wants a notifier with the current temperature
---------------------------------------------------------------------------*/
void CBtnUnits::OnRequestNotifier(CRequestNotifier *pNotifier)
{
   if (pNotifier->m_nClass == UNITS_NOTIFIER)
   {
      if (pNotifier->m_bSend)
      {
         SendUnitsNotifier(m_units);
      }
      else
      {
         PostUnitsNotifier(m_units);
      }
   }
}

/*---------------------------------------------------------------------------
OnClicked   Button was pressed, toggle units    
---------------------------------------------------------------------------*/
void CBtnUnits::OnClicked() 
{
   m_units = (m_units == CELSIUS) ? FAHRENHEIT : CELSIUS;
   SetWindowText((m_units == CELSIUS) ? "C" : "F");
   PostUnitsNotifier(m_units);
}
