/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
TextDegrees.cpp   Static text window showing current temperature 
                  in degrees Celsius or Fahrenheit, depending on
                  the current Units setting.
---------------------------------------------------------------------------*/
#include "stdafx.h"
#include "thermdef.h"
#include "TextDegrees.h"
#include "temperatureNotifier.h"
#include "requestNotifier.h"
#include "unitsNotifier.h"
#include "simerr_notify.h"       // error simulation

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/*---------------------------------------------------------------------------
Message Map
---------------------------------------------------------------------------*/
BEGIN_MESSAGE_MAP(CTextDegrees, CWnd)
	//{{AFX_MSG_MAP(CTextDegrees)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/*---------------------------------------------------------------------------
CTextDegrees  constructor
---------------------------------------------------------------------------*/
CTextDegrees::CTextDegrees()
             :CWnd(), CThermSubscriber()
{
   m_rectWindow.SetRectEmpty();
   m_fCurrTemp = 0.0;
   m_units = CELSIUS;
}

/*---------------------------------------------------------------------------
CTextDegrees  destructor
---------------------------------------------------------------------------*/
CTextDegrees::~CTextDegrees()
{
}

/*---------------------------------------------------------------------------
Init           create the static text control
---------------------------------------------------------------------------*/
HRESULT CTextDegrees::Init(CWnd *pParent, UINT nID)
{
   BOOL bCreated;
   // create the window
   bCreated = CreateEx(WS_EX_STATICEDGE,
                       NULL, "", 
                       WS_CHILD | WS_VISIBLE,
                       m_rectWindow, pParent, nID,
                       0);
   bCreated = SimErrCreateWin(SIMLINE, bCreated, this);
   if (bCreated)
   {
      m_font.CreateStockObject(ANSI_VAR_FONT);
      m_brushLtGray.CreateStockObject(LTGRAY_BRUSH);

      // Subscribe to notifiers and request initial update
      Subscribe(TEMPERATURE_NOTIFIER);
      Subscribe(UNITS_NOTIFIER);
      RequestNotifierPost(TEMPERATURE_NOTIFIER);
      RequestNotifierPost(UNITS_NOTIFIER);
   }
   return (bCreated) ? S_OK : E_OUTOFMEMORY;
}

/*---------------------------------------------------------------------------
OnPaint        WM_PAINT - draw the graph
               draw a gray background then draw the grid lines
---------------------------------------------------------------------------*/
void CTextDegrees::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

   // gray background
   CRect rectInvalid = dc.m_ps.rcPaint;
   dc.FillRect(&rectInvalid, &m_brushLtGray);

   // get the text string and calculate position within window
   char text[8];
   long nCurrTemp = Round(m_fCurrTemp);
   if (m_units == FAHRENHEIT)
   {
      nCurrTemp = Round(ToFahrenheit(m_fCurrTemp));
   }
   sprintf(text, "%d", nCurrTemp);
   CSize size = dc.GetTextExtent(text, strlen(text));
   int x = (m_rectWindow.Width()  - size.cx) / 2;
   int y = (m_rectWindow.Height() - size.cy) / 2;

   // choose color
   COLORREF rgbColor = RGB(0,0,0);  // black

   // set drawing mode, text color, and font and draw the text
   int oldMode = dc.SetBkMode(TRANSPARENT);
	CFont *pOldFont = (CFont *) dc.SelectObject(&m_font);
   COLORREF oldColor = dc.SetTextColor(rgbColor);

   dc.TextOut(x, y, text, strlen(text));

   dc.SetBkMode(oldMode);
   dc.SetTextColor(oldColor);
   dc.SelectObject(pOldFont);
}

/*---------------------------------------------------------------------------
GetDesiredSize increase given width and height as necessary to satisfy 
               minimum size requirements
---------------------------------------------------------------------------*/
void CTextDegrees::GetDesiredSize(int &width, int &height)
{
   HRESULT hr = S_OK;
   CSize   minSize;
	CDC     dc;	
   CFont   font;

   minSize.cx = 100;
   minSize.cy = 24;
   font.CreateStockObject(ANSI_VAR_FONT);
	if (dc.CreateIC("DISPLAY", 0,0,0))
   {
      // First get size of the text
      dc.SelectObject(&font);
      minSize = dc.GetTextExtent("999", 3);
      // Then add a border for the button
      minSize += CSize(24, 6);
   }

   width = max(width, minSize.cx);
   height = minSize.cy;
}

/*---------------------------------------------------------------------------
SetRect        Set the window rectangle
---------------------------------------------------------------------------*/
void CTextDegrees::SetRect(CRect &rect)
{
   if (GetSafeHwnd() && (m_rectWindow != rect))
   {
      MoveWindow(rect, TRUE);
   }
   m_rectWindow = rect;
}


/*---------------------------------------------------------------------------
OnTemperatureNotifier   update the temperature display
---------------------------------------------------------------------------*/
void CTextDegrees::OnTemperatureNotifier(CTemperatureNotifier *pNotifier)
{
   if (m_fCurrTemp != pNotifier->m_celsius)
   {
      m_fCurrTemp = pNotifier->m_celsius;
      DrawCurrentTemperature();
   }
}

/*---------------------------------------------------------------------------
OnUnitsNotifier   change from celsius to fahrenheit
---------------------------------------------------------------------------*/
void CTextDegrees::OnUnitsNotifier(CUnitsNotifier *pNotifier)
{
   if (m_units != pNotifier->m_units)
   {
      m_units = pNotifier->m_units;
      DrawCurrentTemperature();
   }
}

/*---------------------------------------------------------------------------
DrawCurrentTemperature  set the window text to the current temperature
---------------------------------------------------------------------------*/
void CTextDegrees::DrawCurrentTemperature(void)
{
   if (GetSafeHwnd())
   {
      CRect rectClient;
      GetClientRect(&rectClient);
      Invalidate(FALSE);
   }
}
