/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
graph.h   CGraph window class definition
---------------------------------------------------------------------------*/
#ifndef GRAPH_H
#define GRAPH_H
#include "thermSubscriber.h"

typedef CArray<float, float &> CFloatArray;

class CTempScale;

//---------------------------------------------------------------------------
class CGraph : public CWnd, CThermSubscriber
//---------------------------------------------------------------------------
{
public:
   // Construction and Assignment
   CGraph(void);
   virtual ~CGraph(void);

   // Initialization
   HRESULT Init(CWnd *pParent, UINT nID);

   // Size and position
   void SetRect(CRect &rect);
   static void GetDesiredSize(int &width, int &height);

   // Display
   void DrawGridLines(CDC *pDC, CRect &rectInvalid);
   void DrawTemperature(CDC *pDC, CRect &rectInvalid);
   void DrawSetPoint(CDC *pDC, CRect &rectInvalid);

   // Notifiers
   virtual void OnTemperatureNotifier(CTemperatureNotifier *pNotifier);
   virtual void OnUnitsNotifier(CUnitsNotifier *pNotifier);
   virtual void OnSetPointNotifier(CSetPointNotifier *pNotifier);

protected:
   CRect       m_rectWindow;  // client rect in parent's coordinates
   UINT        m_nTimer;      // timer used to update temperature graph
   CTempScale *m_pScale;      // temperature scale object

   float       m_fCurrTemp;   // current temperature
   THERM_UNITS m_units;       // current display units
   float       m_fSetPoint;   // current set point

   CFloatArray m_fTempArray;  // array of recent temperatures
   CFloatArray m_fSPArray;    // array of set points
   int         m_nTempCount;  // number of valid data points
   DWORD       m_dwNextTime;  // next time to do an update
   DWORD       m_dwUpdateInterval;  // time between updates

   CPen        m_penBlack;
   CPen        m_penLtGray;
   CPen        m_penDkGray;
   CPen        m_penWhite;
   CPen        m_penLtBlue;
   CPen        m_penMercury;
   CBrush      m_brushLtGray;

protected:     // MFC generated functions, message maps, etc.
	//{{AFX_VIRTUAL(CGraph)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CGraph)
	afx_msg void OnPaint();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnDestroy();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};
//{{AFX_INSERT_LOCATION}}

#endif  GRAPH_H

