/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
meter.h  CThermometer class definition

This class draws a liquid-mercury thermometer graphically representing
the current temperature.  It is display only and does not respond to 
user input.
---------------------------------------------------------------------------*/
#ifndef METER_H
#define METER_H
#include "thermSubscriber.h"
class CTempScale;

//---------------------------------------------------------------------------
class CThermometer : public CWnd, CThermSubscriber
//---------------------------------------------------------------------------
{
public:
   // Construction and Assignment
   CThermometer(void);
   virtual ~CThermometer(void);

   // Initialization
   HRESULT Init(CWnd *pParent, UINT nID);

   // Size and position
   void SetRect(CRect &rect);
   static void GetDesiredSize(int &width, int &height);
   static int GetMeterWidth(void);

   // Notifiers
   virtual void OnTemperatureNotifier(CTemperatureNotifier *pNotifier);
   virtual void OnUnitsNotifier(CUnitsNotifier *pNotifier);
protected:
   void DrawMercury(CDC *pDC, CRect &rectInvalid);
   void DrawGradationLines(CDC *pDC, CRect &rectInvalid);
   void DrawNumbers(CDC *pDC, CRect &rectInvalid);

protected:
   CRect       m_rectWindow;  // client rect in parent's coordinates
   CRect       m_rectMercury; // the tube that contains the mercury
   CRect       m_rectNumbers; // numeric scale 

   CTempScale *m_pScale;      // temperature scale object
   float       m_fCurrTemp;   // current temperature
   THERM_UNITS m_units;       // current display units

   int         m_gradeLong;   // width of long gradation lines
   int         m_gradeShort;  // width of short gradation lines

   CPen        m_penBlack;
   CPen        m_penDkGray;
   CPen        m_penLtBlue;
   CBrush      m_brushLtGray;
   CBrush      m_brushMercury;
   CFont       m_font;

protected:     // MFC generated functions, message maps, etc.
	//{{AFX_VIRTUAL(CThermometer)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CThermometer)
	afx_msg void OnPaint();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};
//{{AFX_INSERT_LOCATION}}

#endif // METER_H
