/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
requestNotifier.h   Send this notifier to request that a given notifier
                    be sent or posted.
---------------------------------------------------------------------------*/
#ifndef REQUEST_NOTIFIER_H
#define REQUEST_NOTIFIER_H

#include "thermdef.h"
#include "notifier.h"

//---------------------------------------------------------------------------
class CRequestNotifier : public CNotifier
//---------------------------------------------------------------------------
{
public:
   CRequestNotifier(void);
   CRequestNotifier(const CRequestNotifier &src);
   virtual ~CRequestNotifier(void);
   CRequestNotifier &operator=(const CRequestNotifier &notifier);

public:
   NOTIFIER_CLASS       m_nClass;
   CNotifier::PRIORITY  m_nPriority;
   bool                 m_bSend;
};

/************************
 *** INLINE FUNCTIONS ***
 ***********************/

/*---------------------------------------------------------------------------
default constructor
---------------------------------------------------------------------------*/
inline CRequestNotifier::CRequestNotifier(void)
                        :CNotifier(REQUEST_NOTIFIER)
{
   // default is post at priority normal
   m_nClass = NOTIFIER_CLASS(0);
   m_nPriority = PRIORITY_NORMAL;
   m_bSend = false;  
}
/*---------------------------------------------------------------------------
copy constructor
---------------------------------------------------------------------------*/
inline CRequestNotifier::CRequestNotifier(const CRequestNotifier &src)
                      :CNotifier(src)
{
   *this = src;
}
/*---------------------------------------------------------------------------
destructor
---------------------------------------------------------------------------*/
inline CRequestNotifier::~CRequestNotifier(void)
{
}
/*---------------------------------------------------------------------------
Assignment operator
Note: m_pObject ptr is copied, the object itself is not copied.
---------------------------------------------------------------------------*/
inline CRequestNotifier &CRequestNotifier::operator=(const CRequestNotifier &src)
{
   CNotifier::operator=(src);
   m_nClass    = src.m_nClass;  
   m_nPriority = src.m_nPriority;  
   m_bSend     = src.m_bSend;  
   return *this;
}
/*---------------------------------------------------------------------------
synchronous convenience function
sends a notifier to request that a notifier be immediately sent back
---------------------------------------------------------------------------*/
inline HRESULT RequestNotifier(NOTIFIER_CLASS nClass)
{
   HRESULT hr = E_OUTOFMEMORY;
   CRequestNotifier *pNotifier = new CRequestNotifier();
   if (pNotifier)
   {
      pNotifier->m_nClass = nClass;
      pNotifier->m_bSend  = true;  
      pNotifier->Send();
   }
   return hr;
}
/*---------------------------------------------------------------------------
asynchronous convenience function
posts a notifier to request a posted notifier
---------------------------------------------------------------------------*/
inline HRESULT RequestNotifierPost(NOTIFIER_CLASS nClass, 
                                   CNotifier::PRIORITY nPriority = CNotifier::PRIORITY_NORMAL)
{
   HRESULT hr = E_OUTOFMEMORY;
   CRequestNotifier *pNotifier = new CRequestNotifier();
   if (pNotifier)
   {
      pNotifier->m_nClass = nClass;
      pNotifier->m_nPriority = nPriority;
      pNotifier->Post();
   }
   return hr;
}

#endif // THERM_NOTIFIER_H

