/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
scale.h   CTempScale class definition
          Calculates the scale of temperature windows based on
          current units and window height.
---------------------------------------------------------------------------*/
#ifndef SCALE_H
#define SCALE_H
#include "thermSubscriber.h"

//---------------------------------------------------------------------------
class CTempScale
//---------------------------------------------------------------------------
{
public:
   // Construction and Assignment
   CTempScale(void);
   virtual ~CTempScale(void);

   // Accessors
   CSize GetCellSize(void)    { return m_cellSize; }
   float GetLoTemp(void)      { return m_loTemp;   }
   float GetHiTemp(void)      { return m_hiTemp;   }
   int   GetHeight(void)      { return m_nScaleHeight; }
   int   GetScale(void)       { return m_nScale;   }
   int   GetInset(int nHeight) { return (nHeight - m_nScaleHeight) / 2; }

   // Calculation functions
   long CelsiusToPixels(float fTemperature);
   float PixelsToCelsius(long nPixels);
   void CalcScaleInterval(int nWndHeight, THERM_UNITS units);

   // Static calculation functions
   static CSize GetTextCellSize(void);
   static long RoundToNearestMultiple(float fInput, int nDivisor);

protected:
   THERM_UNITS       m_units;    // temperature units
   CSize             m_cellSize; // grid scale height and width
   float             m_loTemp;   // low and high
   float             m_hiTemp;   //   temperatures in Celsius
   int               m_nScaleHeight;  // thermometer/graph height
   int               m_nScale;   // scale interval in degrees
};

#endif  SCALE_H

