/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
setpointNotifier.h   Notify subscribers of a change in the thermostat setpoint
---------------------------------------------------------------------------*/
#ifndef SETPOINT_NOTIFIER_H
#define SETPOINT_NOTIFIER_H

#include "thermdef.h"
#include "notifier.h"

//---------------------------------------------------------------------------
class CSetPointNotifier : public CNotifier
//---------------------------------------------------------------------------
{
public:
   CSetPointNotifier(void);
   CSetPointNotifier(const CSetPointNotifier &src);
   virtual ~CSetPointNotifier(void);
   CSetPointNotifier &operator=(const CSetPointNotifier &notifier);

public:
   float       m_celsius;   // the setpoint is always celsius
};

/************************
 *** INLINE FUNCTIONS ***
 ***********************/

/*---------------------------------------------------------------------------
default constructor
---------------------------------------------------------------------------*/
inline CSetPointNotifier::CSetPointNotifier(void)
                         :CNotifier(SETPOINT_NOTIFIER)
{
   m_celsius = 100.0;
}
/*---------------------------------------------------------------------------
copy constructor
---------------------------------------------------------------------------*/
inline CSetPointNotifier::CSetPointNotifier(const CSetPointNotifier &src)
                         :CNotifier(src)
{
   *this = src;
}
/*---------------------------------------------------------------------------
destructor
---------------------------------------------------------------------------*/
inline CSetPointNotifier::~CSetPointNotifier(void)
{
}
/*---------------------------------------------------------------------------
Assignment operator
Note: m_pObject ptr is copied, the object itself is not copied.
---------------------------------------------------------------------------*/
inline CSetPointNotifier &CSetPointNotifier::operator=(const CSetPointNotifier &src)
{
   CNotifier::operator=(src);
   m_celsius = src.m_celsius;  
   return *this;
}
/*---------------------------------------------------------------------------
convenience function -- send a notification of a temperature change
---------------------------------------------------------------------------*/
inline HRESULT SendSetPointNotifier(float celsius)
{
   HRESULT hr = E_OUTOFMEMORY;
   CSetPointNotifier *pNotifier = new CSetPointNotifier();
   if (pNotifier)
   {
      pNotifier->m_celsius = celsius;
      pNotifier->Send();
   }
   return hr;
}
/*---------------------------------------------------------------------------
convenience function -- post a notification of a temperature change
---------------------------------------------------------------------------*/
inline HRESULT PostSetPointNotifier(float celsius, 
                                    CNotifier::PRIORITY nPriority = CNotifier::PRIORITY_NORMAL)
{
   HRESULT hr = E_OUTOFMEMORY;
   CSetPointNotifier *pNotifier = new CSetPointNotifier();
   if (pNotifier)
   {
      pNotifier->m_celsius = celsius;
      pNotifier->Post(nPriority);
   }
   return hr;
}

#endif // SETPOINT_NOTIFIER_H

