/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
temperatureNotifier.h   Notify subscribers of a temperature change
---------------------------------------------------------------------------*/
#ifndef TEMPERATURE_NOTIFIER_H
#define TEMPERATURE_NOTIFIER_H

#include "thermdef.h"
#include "notifier.h"

//---------------------------------------------------------------------------
class CTemperatureNotifier : public CNotifier
//---------------------------------------------------------------------------
{
public:
   CTemperatureNotifier(void);
   CTemperatureNotifier(const CTemperatureNotifier &src);
   virtual ~CTemperatureNotifier(void);
   CTemperatureNotifier &operator=(const CTemperatureNotifier &notifier);

public:
   float       m_celsius;   // this temperature is always celsius
};

/************************
 *** INLINE FUNCTIONS ***
 ***********************/

/*---------------------------------------------------------------------------
default constructor
---------------------------------------------------------------------------*/
inline CTemperatureNotifier::CTemperatureNotifier(void)
                            :CNotifier(TEMPERATURE_NOTIFIER)
{
   m_celsius = 100.0;
}
/*---------------------------------------------------------------------------
copy constructor
---------------------------------------------------------------------------*/
inline CTemperatureNotifier::CTemperatureNotifier(const CTemperatureNotifier &src)
                            :CNotifier(src)
{
   *this = src;
}
/*---------------------------------------------------------------------------
destructor
---------------------------------------------------------------------------*/
inline CTemperatureNotifier::~CTemperatureNotifier(void)
{
}
/*---------------------------------------------------------------------------
Assignment operator
Note: m_pObject ptr is copied, the object itself is not copied.
---------------------------------------------------------------------------*/
inline CTemperatureNotifier &CTemperatureNotifier::operator=(const CTemperatureNotifier &src)
{
   CNotifier::operator=(src);
   m_celsius = src.m_celsius;  
   return *this;
}
/*---------------------------------------------------------------------------
convenience function -- send a notification of a temperature change
---------------------------------------------------------------------------*/
inline HRESULT SendTemperatureNotifier(float celsius)
{
   HRESULT hr = E_OUTOFMEMORY;
   CTemperatureNotifier *pNotifier = new CTemperatureNotifier();
   if (pNotifier)
   {
      pNotifier->m_celsius = celsius;
      pNotifier->Send();
   }
   return hr;
}
/*---------------------------------------------------------------------------
convenience function -- post a notification of a temperature change
---------------------------------------------------------------------------*/
inline HRESULT PostTemperatureNotifier(float celsius, 
                                       CNotifier::PRIORITY nPriority = CNotifier::PRIORITY_NORMAL)
{
   HRESULT hr = E_OUTOFMEMORY;
   CTemperatureNotifier *pNotifier = new CTemperatureNotifier();
   if (pNotifier)
   {
      pNotifier->m_celsius = celsius;
      pNotifier->Post(nPriority);
   }
   return hr;
}

#endif // TEMPERATURE_NOTIFIER_H

