/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
thermDispatcher.cpp  Override CDispatcher to dispatch the notifier
                     subclasses defined for the thermostat.

The thermostat application defined several notifier subclasses that 
provide data on things like temperature, setpoints, etc.

CThermSubscriber subclasses CSubscriber to provide virtual functions that
respond to these notifiers, like OnTemperatureNotifier, OnSetpointNotifier,
etc.

This dispatcher subclasses recognizes the thermostat notifiers and 
dispatches them to the corresponding CThermSubscriber OnXXXNotifier functions.
---------------------------------------------------------------------------*/
#include "stdafx.h"
#include "temperatureNotifier.h"
#include "thermSubscriber.h"
#include "thermDispatcher.h"

/*---------------------------------------------------------------------------
DispatchToOne  Dispatch one notifier to one subscriber
---------------------------------------------------------------------------*/
HRESULT CThermDispatcher::DispatchToOne(CNotifier *pNotifier,
                                        CSubscriber *pBaseSubscriber)
{
   CThermSubscriber *pSubscriber = (CThermSubscriber *) pBaseSubscriber;

   switch (pNotifier->GetClass())
   {
   case REQUEST_NOTIFIER:
      pSubscriber->OnRequestNotifier((CRequestNotifier*) pNotifier);
      break;

   case TEMPERATURE_NOTIFIER:
      pSubscriber->OnTemperatureNotifier((CTemperatureNotifier*) pNotifier);
      break;

   case UNITS_NOTIFIER:
      pSubscriber->OnUnitsNotifier((CUnitsNotifier*) pNotifier);
      break;

   case SETPOINT_NOTIFIER:
      pSubscriber->OnSetPointNotifier((CSetPointNotifier*) pNotifier);
      break;

   default:
      ASSERT(false);
   }

   return S_OK;
}

