/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
thermdef.h   Definitions shared by all the classes in the thermostat 
             control.
---------------------------------------------------------------------------*/
#ifndef THERMDEF_H
#define THERMDEF_H

// For the example in the notifier article, define several releases
#define RELEASE_2          // new feature: temperature graph
#define RELEASE_3          // new feature: setpoint slider

// Notifier Classes
enum NOTIFIER_CLASS
{
   REQUEST_NOTIFIER,       // sent to request an updated notifier
   TEMPERATURE_NOTIFIER,   // sent when the temperature changes
   UNITS_NOTIFIER,         // sent when the temperature units change
   SETPOINT_NOTIFIER       // sent when thermostat setpoint changes
};

// Communication with temperature sensor thread
#define TEMPERATURE_THREAD (UINT(1000))
#define THREAD_FAILED      (WPARAM(1000))

// Display definitions
#define GRID_INTERVAL           5      // pixels between vertical grid lines 
#define MIN_METER_HEIGHT       240     // minimum height and width 
#define MIN_METER_WIDTH         30     //   of thermometer in pixels 
#define MIN_SLIDER_WIDTH        20     // min width of setpoint slider
#define LO_TEMP                -40     // lowest and highest temperatures
#define HI_TEMP                 50     //   displayed in thermometer
#define RGB_DARK_GRAY       (RGB(128,128,128))
#define RGB_LIGHT_GRAY      (RGB(192,192,192))
#define RGB_GRIDLINE_COLOR  (RGB( 32, 32,192))
#define RGB_MERCURY_COLOR   (RGB(192, 32, 32))
enum THERM_UNITS { CELSIUS, FAHRENHEIT };

/***********************
*** INLINE FUNCTIONS ***
***********************/

/*---------------------------------------------------------------------------
ToFahrenheit   Convert temperature in Celsius to Fahrenheit
---------------------------------------------------------------------------*/
inline float ToFahrenheit(float celsius)
{
   return float(celsius * 1.8 + 32.0);
}

/*---------------------------------------------------------------------------
ToCelsius      Convert temperature in Fahrenheit to Celsius
---------------------------------------------------------------------------*/
inline float ToCelsius(float fFahrenheit)
{
   return float((fFahrenheit - 32.0) / 1.8);
}

/*---------------------------------------------------------------------------
Round          Round a float to the nearest integer value
---------------------------------------------------------------------------*/
inline int Round(float fValue)
{
   if (fValue < 0)
   {
      fValue -= 0.5;
   }
   else
   {
      fValue += 0.5;
   }
   return int(fValue);
}



/***********************
*** GLOBAL FUNCTIONS ***
***********************/
class CTempScale;
CTempScale *GetAppScale(void);

#endif  THERMDEF_H

