/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
THERMOSTAT.CPP    CWinApp subclass for the thermostat.

This is mostly MFC boilerplate
---------------------------------------------------------------------------*/
#include "stdafx.h"
#include "thermostat.h"
#include "thermdef.h"
#include "MainFrm.h"
#include "doc.h"
#include "view.h"
#include "thermDispatcher.h"
#include "scale.h"
#include "about.h"
#include "simerr_notify.h"       // error simulation

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/*---------------------------------------------------------------------------
Message Map
---------------------------------------------------------------------------*/
BEGIN_MESSAGE_MAP(CThermApp, CWinApp)
	//{{AFX_MSG_MAP(CThermApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/*---------------------------------------------------------------------------
CThermApp constructor
---------------------------------------------------------------------------*/
CThermApp::CThermApp()
{
   m_pDispatcher = 0;
}

/*---------------------------------------------------------------------------
The one and only CThermApp object
---------------------------------------------------------------------------*/
CThermApp theApp;

/*---------------------------------------------------------------------------
InitInstance  create dispatcher and temperature scale calculator
              create main window and position it
---------------------------------------------------------------------------*/
BOOL CThermApp::InitInstance()
{
   // initialize error simulation testing -- for testing error handlers
   //    error simulation will not be compiled unless SIMERR is defined
   SimErrCustomInit(SIMERR_TIME_SEED, SIMERR_RANDOM);

   BOOL bOK = TRUE;

   // create and initialize the notifier dispatcher
   m_pDispatcher = new CThermDispatcher();
   m_pDispatcher = SimErrNew(SIMLINE, m_pDispatcher);
   bOK = (m_pDispatcher) ? TRUE : FALSE;

   if (bOK)
   {
      HRESULT hrDispInit = m_pDispatcher->Init();
      hrDispInit = SimErrResult(SIMLINE, hrDispInit, E_OUTOFMEMORY, 6, .2);
      if (FAILED(hrDispInit))
      {
         bOK = FALSE;
      }
   }

   // create the object that calculates pixel positions for temperatures
   //   within the thermometer and the temperature graph
   if (bOK)
   {
      m_pScale = new CTempScale();
      m_pScale = SimErrNew(SIMLINE, m_pScale, 10, .1);
      if (!m_pScale)
      {
         bOK = FALSE;
      }
   }

   if (bOK)
   {
   #ifdef _AFXDLL
	   Enable3dControls();			// Call this when using MFC in a shared DLL
   #else
	   Enable3dControlsStatic();	// Call this when linking to MFC statically
   #endif

	   // Change the registry key under which our settings are stored.
	   SetRegistryKey(_T("Thermostat"));

	   LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	   // Register document templates

	   CSingleDocTemplate* pDocTemplate;
	   pDocTemplate = new CSingleDocTemplate(
		   IDR_MAINFRAME,
		   RUNTIME_CLASS(CDoc),
		   RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		   RUNTIME_CLASS(CThermView));
      pDocTemplate = SimErrNew(SIMLINE, pDocTemplate, 5, .2);
	   AddDocTemplate(pDocTemplate);

   }
   if (bOK)
   {
	   // Parse command line for standard shell commands, DDE, file open
	   CCommandLineInfo cmdInfo;
	   ParseCommandLine(cmdInfo);

	   // Dispatch commands specified on the command line
	   bOK = ProcessShellCommand(cmdInfo);
      bOK = SimErrZero(SIMLINE, bOK, 4, .5);
   }
   if (bOK)
   {
      SetMainWindowPlacement();
	   m_pMainWnd->ShowWindow(SW_SHOW);
	   m_pMainWnd->UpdateWindow();
   }
   if (!bOK)
   {
      AfxMessageBox("Unable to start Thermostat application");
   }

	return bOK;
}

/*---------------------------------------------------------------------------
ExitInstance
---------------------------------------------------------------------------*/
int CThermApp::ExitInstance() 
{
   if (m_pDispatcher)
   {
      delete m_pDispatcher;
      m_pDispatcher = 0;
   }
   if (m_pScale)
   {
      delete m_pScale;
      m_pScale = 0;
   }
   SimErrExit(true);
	return CWinApp::ExitInstance();
}

/*---------------------------------------------------------------------------
SetMainWindowPlacement  position window when first created
---------------------------------------------------------------------------*/
void CThermApp::SetMainWindowPlacement(void)
{
	WINDOWPLACEMENT winPlace;
   CRect           deskRect;
	CWnd *pDesktop = m_pMainWnd->GetDesktopWindow();
	pDesktop->GetWindowRect(deskRect);

   // Calculate size of application window
   int width;
   int height;
   CThermView::GetSize(width, height);
   width  += 2 * GetSystemMetrics(SM_CXFRAME) + 4;
   height += GetSystemMetrics(SM_CYCAPTION)
             + GetSystemMetrics(SM_CYMENU)
			    + 2 * GetSystemMetrics(SM_CYFRAME) + 4;
#ifdef RELEASE_2
   width += 100;  // add room for the graph
#endif

   // center window vertically
   int y = (deskRect.Height() - height) / 2;
   // center window horizontally
   int x = (deskRect.Width() - width) / 2;
   CRect winRect(x, y, x + width, y + height);

   memset(&winPlace, 0, sizeof(WINDOWPLACEMENT));
	winPlace.length = sizeof(WINDOWPLACEMENT);
	winPlace.rcNormalPosition = winRect;
   winPlace.showCmd = SW_RESTORE;
	m_pMainWnd->SetWindowPlacement(&winPlace);
}

/*---------------------------------------------------------------------------
OnAppAbout  display the about box
---------------------------------------------------------------------------*/
void CThermApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/*---------------------------------------------------------------------------
GetAppScale  return pointer to the temperature scaling object
---------------------------------------------------------------------------*/
CTempScale *GetAppScale(void) 
{
   return ((CThermApp *) AfxGetApp())->GetScale();
}
