/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
unitsNotifier.h   Notify subscribers of a change in the units used
                  to display the temperature.
                  Units are either Fahrenheit or Celsius
---------------------------------------------------------------------------*/
#ifndef UNITS_NOTIFIER_H
#define UNITS_NOTIFIER_H

#include "thermdef.h"
#include "notifier.h"

//---------------------------------------------------------------------------
class CUnitsNotifier : public CNotifier
//---------------------------------------------------------------------------
{
public:
   CUnitsNotifier(void);
   CUnitsNotifier(const CUnitsNotifier &src);
   virtual ~CUnitsNotifier(void);
   CUnitsNotifier &operator=(const CUnitsNotifier &notifier);

public:
   THERM_UNITS m_units;
};

/************************
 *** INLINE FUNCTIONS ***
 ***********************/

/*---------------------------------------------------------------------------
default constructor
---------------------------------------------------------------------------*/
inline CUnitsNotifier::CUnitsNotifier(void)
                      :CNotifier(UNITS_NOTIFIER)
{
   m_units = FAHRENHEIT;
}
/*---------------------------------------------------------------------------
copy constructor
---------------------------------------------------------------------------*/
inline CUnitsNotifier::CUnitsNotifier(const CUnitsNotifier &src)
                      :CNotifier(src)
{
   *this = src;
}
/*---------------------------------------------------------------------------
destructor
---------------------------------------------------------------------------*/
inline CUnitsNotifier::~CUnitsNotifier(void)
{
}
/*---------------------------------------------------------------------------
Assignment operator
Note: m_pObject ptr is copied, the object itself is not copied.
---------------------------------------------------------------------------*/
inline CUnitsNotifier &CUnitsNotifier::operator=(const CUnitsNotifier &src)
{
   CNotifier::operator=(src);
   m_units  = src.m_units;  
   return *this;
}
/*---------------------------------------------------------------------------
convenience function -- send a notification of a temperature units change
---------------------------------------------------------------------------*/
inline HRESULT SendUnitsNotifier(THERM_UNITS units)
{
   HRESULT hr = E_OUTOFMEMORY;
   CUnitsNotifier *pNotifier = new CUnitsNotifier();
   if (pNotifier)
   {
      pNotifier->m_units = units;
      pNotifier->Send();
   }
   return hr;
}
/*---------------------------------------------------------------------------
convenience function -- post a notification of a temperature units change
---------------------------------------------------------------------------*/
inline HRESULT PostUnitsNotifier(THERM_UNITS nUnits, 
                                 CNotifier::PRIORITY nPriority = CNotifier::PRIORITY_NORMAL)
{
   HRESULT hr = E_OUTOFMEMORY;
   CUnitsNotifier *pNotifier = new CUnitsNotifier();
   if (pNotifier)
   {
      pNotifier->m_units = nUnits;
      pNotifier->Post(nPriority);
   }
   return hr;
}

#endif // THERM_NOTIFIER_H

