/*---------------------------------------------------------------------------
         Copyright 1997    On Target Software    All Rights Reserved

  Dave Pomerantz, Nov 1997.  Updates available at http://www.targetsoft.com
-----------------------------------------------------------------------------
ThermView.h   CThermView class implementation

Create and position all the display objects:

RELEASE 1
   thermometer    bar of mercury, numeric scale and gradation lines
   text degrees   text field with temperature degrees in current units
   units button   switch units between Celsius and Fahrenheit

RELEASE 2
   graph          graph of temperature over time

RELEASE 3
   slider         control that adjusts setpoint
   text setpoint  text field with setpoint degrees in current units
---------------------------------------------------------------------------*/
#ifndef THERMVIEW_H
#define THERMVIEW_H

#include "thermdef.h"
#include "thermSubscriber.h"

// forward declarations
class CThermometer;
class CTextDegrees;
class CBtnUnits;
class CGraph;
class CSetPointSlider;
class CTextSetPoint;
class CUnitsNotifier;

//---------------------------------------------------------------------------
class CThermView : public CView, CThermSubscriber
//---------------------------------------------------------------------------
{
protected:
	CThermView();
	~CThermView();
   HRESULT Init(void);
   HRESULT InitSensor(void);
   HRESULT InitDisplay(void);
   void Resize(int &width, int &height);

   // Notifier handlers
   virtual void OnUnitsNotifier(CUnitsNotifier *pNotifier);

protected:
   enum { FIRST_WINDOW_ID = 1001 }; // ID of first child window created

   bool            m_bInitialized;
         // display windows
   CThermometer   *m_pThermometer;  // analog meter
   CTextDegrees   *m_pTxtDegrees;   // static text window
   CBtnUnits      *m_pBtnUnits;     // fahrenheit/celsius
#ifdef RELEASE_2
   CGraph         *m_pGraph;
#endif
#ifdef RELEASE_3
   CSetPointSlider *m_pSlider;
   CTextSetPoint   *m_pTextSetPoint;
#endif

         // temperature sensor
   CWinThread     *m_pTemperatureThread;  // sensor object
   CEvent          m_hEvent;           // communicates with sensor thread
   THERM_UNITS     m_units;            // temperature units
   CRect           m_thermRect;     

public:
   static void GetSize(int &width, int &height);
	CDoc* GetDocument();

public:  // Overrides
	//{{AFX_VIRTUAL(CThermView)
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual void OnInitialUpdate();
	//}}AFX_VIRTUAL

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:     // MFC generated functions, message maps, etc.
	//{{AFX_MSG(CThermView)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
	DECLARE_DYNCREATE(CThermView)
};

#ifndef _DEBUG  // debug version in view.cpp
inline CDoc* CThermView::GetDocument()
   { return (CDoc*)m_pDocument; }
#endif

//{{AFX_INSERT_LOCATION}}

#endif // THERMVIEW_H
